import React from 'react';
import PropTypes from 'prop-types';
import LayersDetails from './LayersDetails';
import CustomFieldsService from './api/CustomFieldsService';
import CorgeeStorageHelper from './globals/CorgeeStorageHelper';
import HeaderDetails from './HeaderDetails'
import RenderInBody from "./globals/RenderInBody";
// import { RenderToBody } from "./globals/AppendToBody";
// import './styles/styles.css';

// import 'react-select/dist/react-select.css';
import 'bootstrap/dist/css/bootstrap.css';
// import 'bootstrap';
// import 'bootstrap/js/modal';
// import 'bootstrap/js/dropdown';
// import 'bootstrap/js/tooltip';
import './styles/glyphs/material-icons.css';
import './styles/glyphs/glyphs.css';
import './styles/glyphs/glyphs-news/glyphs2.css';

import './ConversationSummernoteStyles.css';

// import './ComponentHelpers.css';
import './CustomFields.css';

class CustomFields extends React.Component {
    constructor(props) {
        super(props);

        let apiConfig = CorgeeStorageHelper.getApiConfig(props.config);

        this.state = {
            disableAutoSave: props.config.disableAutoSave,
            EditorEnabled: false,
            canEdit: props.config.canEdit,
            isNew: props.config.isNew,
            showHeader: (props.config.showHeader !== undefined) ? props.config.showHeader : true,
            customFieldsbyLayer: [],
            config: apiConfig,
            requestConfig: {
                projectId: this.props.projectId,
                configValueId: this.props.configValueId,
                coreTaskId: (this.props.item !== undefined) ? this.props.item.CoreTaskId : null,
                personId: (this.props.item !== undefined) ? this.props.item.PersonId : null,
                isDeleted: false,
                api: {
                    url: apiConfig.apiUrl,
                    token: apiConfig.authToken
                }
            },
            initialLoadData: false,
            showDetails: true,
            layersToHide: [],
            isRecycleBin: (window.location.href.indexOf("RecycleBin") > -1) ? true : false,
            projectNameConfig: CorgeeStorageHelper.getProjectNameConfig(),
            containerSelector: undefined,
            isProjectView: (window.location.href.indexOf("ProjectDetails") > -1) ? true : false,
            groups: props.groups,
            highlight: false
        }

        this.UpdateCustomFieldsByLayer = this.UpdateCustomFieldsByLayer.bind(this);
        this.retrieveCustomDetailsData = this.retrieveCustomDetailsData.bind(this);
        this.getCustomFieldsData = this.getCustomFieldsData.bind(this);
        this.toogleShowDetails = this.toogleShowDetails.bind(this);
        this.reloadCustomField = this.reloadCustomField.bind(this);
        this.resetLayersToHide = this.resetLayersToHide.bind(this);
        this.reloadByLayersToHideStorageChange = this.reloadByLayersToHideStorageChange.bind(this);
        this.sendDataToCallback = this.sendDataToCallback.bind(this);
        this.validateChanges = this.validateChanges.bind(this);
        this.onUpdateConversation = this.onUpdateConversation.bind(this);
        this.onChangeContainer = this.onChangeContainer.bind(this);
        this.hideDetails = this.hideDetails.bind(this);
        this.showDetails = this.showDetails.bind(this);
        this.onRecoverFieldsValues = this.onRecoverFieldsValues.bind(this);
        this.onChangedItemType = this.onChangedItemType.bind(this);
        this.onUpdateLayersConversation = this.onUpdateLayersConversation.bind(this);
        this.onUpdateDetails = this.onUpdateDetails.bind(this);
        this.validateBottomScroll = this.validateBottomScroll.bind(this);
        this.validateBottomScrollDashboard = this.validateBottomScrollDashboard.bind(this);
        this.highlightCustomFields = this.highlightCustomFields.bind(this);
    }

    onChangedItemType(newItemType) {
        let requestConfigCopy = Object.assign({}, this.state.requestConfig);
        requestConfigCopy.configValueId = newItemType.ConfigValueId

        this.setState({ requestConfig: requestConfigCopy });

        if (this.state.isNew === true) {
            CustomFieldsService.getEmptyCustomFieldsByConfigValueId(requestConfigCopy).then(function (response) {
                this.retrieveCustomDetailsData(response);

                if (this.refs.convHeaderDetails) {
                    this.refs.convHeaderDetails.reloadLayers();
                }

            }.bind(this)).catch((error) => {
                console.warn("Api Error", error);
            });;

            if (this.refs.convHeaderDetails) {
                this.refs.convHeaderDetails.onChangedItemType(newItemType);
            }
        }
    }

    onRecoverFieldsValues(dataFieldsValues) {
        if (dataFieldsValues !== undefined && dataFieldsValues !== null && dataFieldsValues.length > 0) {
            this.setState({ customFieldsbyLayer: dataFieldsValues })
        }
    }

    onChangeContainer(containerSelector) {
        this.setState({ containerSelector: containerSelector });
    }

    onUpdateConversation(conversation) {
        let requestConfigCopy = Object.assign({}, this.state.requestConfig);
        requestConfigCopy.projectId = conversation.ProjectId;
        requestConfigCopy.coreTaskId = conversation.CoreTaskId;

        this.setState({
            customFieldsbyLayer: conversation.layerCustomFields,
            requestConfig: requestConfigCopy,
            conversationUpdated: conversation,
            groups: conversation.ConversationGroups
        });

        if (this.refs.convHeaderDetails) {
            this.refs.convHeaderDetails.updateConversation(conversation);
        }
    }

    onUpdateLayersConversation(conversation) {
        this.setState({ conversationUpdated: conversation });

        if (this.refs.convHeaderDetails) {
            this.refs.convHeaderDetails.onUpdateLayersConversation(conversation.ConversationLayers);
        }
    }

    onUpdateDetails(layerCustomFields) {
        let conversationUpdatedCopy = Object.assign({}, this.state.conversationUpdated);
        conversationUpdatedCopy.layerCustomFields = layerCustomFields;

        this.setState({ customFieldsbyLayer: layerCustomFields, conversationUpdated: conversationUpdatedCopy });
    }

    validateBottomScroll() {
        let obj = $(this.refs.customFieldsContainer).closest('.layer-custom-fields-container');
        let top = $(obj).scrollTop();
        let height = $(this.refs.customFieldsContainer).height() - $(obj).height() + 12;

        if (top >= height) {
            $(obj).scrollTop(height - 5);
        }
    }

    validateBottomScrollDashboard() {
        if (this.refs.customFieldsContainer) {
            let obj = $(this.refs.customFieldsContainer).closest('.sortableDashboard');
            let top = $(obj).scrollTop();
            let height = $(this.refs.customFieldsContainer).closest('.sortableDashboard')[0].scrollHeight - $(obj).height() - 12;
            if (top >= height) {
                $(obj).scrollTop(height - 5);
            }
        }
    }

    componentDidMount() {
        const { item, config } = this.props;
        if (item !== undefined && item !== null && config !== undefined && config.callApi === false) {
            let layerCustomFieldsList = item.layerCustomFields;
            if (item.LayerCustomFields !== undefined && item.LayerCustomFields !== null) {
                layerCustomFieldsList = item.LayerCustomFields;
            }
            if (layerCustomFieldsList !== undefined && layerCustomFieldsList !== null) {
                let data = [];
                for (var x in layerCustomFieldsList) {
                    if (layerCustomFieldsList[x] !== undefined && layerCustomFieldsList[x] !== null) {
                        data.push(layerCustomFieldsList[x]);
                    }
                }

                this.setState({ customFieldsbyLayer: data });
                this.sendDataToCallback(data);
            }
        }

        this.setState({ layersToHide: CorgeeStorageHelper.getLayersToHide() });
    }

    resetLayersToHide() {
        this.setState({ layersToHide: CorgeeStorageHelper.getLayersToHide() });
    }

    reloadByLayersToHideStorageChange(e) {
        if (e.key === "C2_LayersToHide" && e.oldValue !== e.newValue) {
            this.setState({ layersToHide: CorgeeStorageHelper.getLayersToHide() });
        }
    }

    componentWillReceiveProps(nextProps, nextState) {
        this.setState({ layersToHide: CorgeeStorageHelper.getLayersToHide() });

        if (nextProps.item !== undefined
            && nextProps.item != null
            && nextProps.item.layerCustomFields !== undefined
            && nextProps.item.layerCustomFields !== null
            && nextProps.item.layerCustomFields.length > 0
            && this.validateChanges(nextProps, this.state)) {

            this.setState({ customFieldsbyLayer: nextProps.item.layerCustomFields });
        }

        if ((this.state.groups === null || (this.state.groups !== undefined && this.state.groups.length === 0))
            && nextProps.groups !== null && nextProps.groups !== undefined && nextProps.groups.length > 0) {
            this.setState({ groups: nextProps.groups });
        }

        if (this.props.projectId !== nextProps.projectId && this.state.showDetails === true) {
            this.setState((requestConfig, nextProps) => {
                requestConfig.requestConfig.projectId = nextProps.projectId;
                return requestConfig;
            });

            setTimeout(function () {
                this.getCustomFieldsData(nextProps, true);
            }.bind(this));
        }
        if(nextProps.item.isAddLayertoCard){
            this.onUpdateLayersConversation(nextProps.item);
        }
        if(nextProps.item.isFromConvertTo){
            this.onUpdateConversation(nextProps.item);
            this.onUpdateLayersConversation(nextProps.item);
            this.reloadCustomField(true);
        }
    }

    highlightCustomFields() {
        let this_ = this;
        this.setState({ highlight: true });
        setTimeout(function () {
            this_.setState({ highlight: false });
        }, 2000);
    }

    validateChanges(nextProps, state) {
        let applyUpdate = false;
        if (state.customFieldsbyLayer !== undefined && state.customFieldsbyLayer !== null) {
            if (nextProps.item.layerCustomFields.length !== state.customFieldsbyLayer.length) {
                return applyUpdate = true;
            }

            $.each(nextProps.item.layerCustomFields, function (index, nextLayer) {
                $.each(state.customFieldsbyLayer, function (index, layer) {
                    if (layer !== undefined && nextLayer !== undefined && layer.LayerId === nextLayer.LayerId && JSON.stringify(layer) !== JSON.stringify(nextLayer)) {
                        applyUpdate = true;
                    }
                });
            });
        }
        return applyUpdate;
    }

    componentWillMount() {
        const { config, getForceReloadDetailsFn } = this.props;

        if (CorgeeStorageHelper.isAuthenticated(config)) {

            setTimeout(function () {
                this.getCustomFieldsData(config, true);
                
                if (getForceReloadDetailsFn && typeof getForceReloadDetailsFn == 'function') {
                    getForceReloadDetailsFn(this.getCustomFieldsData, config);
                }
            }.bind(this));
        } else {
            console.error("CustomFields React Component: You have not authenticated.");
        }
    }

    componentWillUnmount() {
    }

    getCustomFieldsData(config, isReload) {
        const { item, isProjectTitle } = this.props;
        const { isNew, requestConfig, conversationUpdated } = this.state;

        let callApi = (config.callApi !== undefined && config.callApi !== null && config.callApi === false) ? false : true;

        let itemConv = item;
        if (conversationUpdated !== undefined && conversationUpdated !== null) {
            itemConv = conversationUpdated;
        }
        if (callApi && itemConv !== undefined) {
            if (isNew) {
                CustomFieldsService.getEmptyCustomFieldsByConfigValueId(requestConfig).then(function (response) {
                    this.retrieveCustomDetailsData(response, config.highlight);
                }.bind(this)).catch((error) => {
                    console.warn("Api Error", error);
                });;
            }
            else {

                if (itemConv.PersonId != null && itemConv.PersonId > 0 && itemConv.ConversationId > 0) {
                    CustomFieldsService.getPersonCustomFieldsByItemId(requestConfig).then(function (response) {
                        this.retrieveCustomDetailsData(response, config.highlight);
                    }.bind(this)).catch((error) => {
                        console.warn("Api Error", error);
                    });
                } else {
                    if ((itemConv.CoreTaskId > 0 || itemConv.IsProjectConversation || itemConv.ConversationId > 0)) {
                        if (itemConv.IsProjectConversation || isProjectTitle) {
                            CustomFieldsService.getProjectCustomFieldsByItemId(requestConfig).then(function (response) {
                                this.retrieveCustomDetailsData(response, config.highlight);
                            }.bind(this)).catch((error) => {
                                console.warn("Api Error", error);
                            });
                        } else if (itemConv.CoreTaskId > 0) {
                            CustomFieldsService.getCustomFieldsByItemId(requestConfig).then(function (response) {
                                this.retrieveCustomDetailsData(response, config.highlight);
                            }.bind(this)).catch((error) => {
                                console.warn("Api Error", error);
                            });
                        } else if (itemConv.ConversationId > 0) {
                            CustomFieldsService.getLayersByConfigValueId(requestConfig).then(function (response) {
                                this.retrieveCustomDetailsData(response, config.highlight);
                            }.bind(this)).catch((error) => {
                                console.warn("Api Error", error);
                            });
                        }
                    }
                }
            }
        }
    }

    retrieveCustomDetailsData(customFieldsResponse, highlight) {
        this.setState({ customFieldsbyLayer: customFieldsResponse, initialLoadData: true, loadInProgress: false });
        
        if (highlight) {
            this.highlightCustomFields();
        }

        this.sendDataToCallback(customFieldsResponse);
        setTimeout(function () {
            if (this.props.config.onRecoverFieldsValues) {
                let convCopy = Object.assign({}, this.props.item);
                convCopy.layerCustomFields = customFieldsResponse;

                this.props.config.onRecoverFieldsValues(convCopy);
            }
        }.bind(this));
    }

    sendDataToCallback(data) {
        if (this.props.config.onLoadDataCallback) {
            let dashboardGroupId = (this.props.item !== undefined) ? this.props.item.dashboardGroupId : null;
            let conversationId = (this.props.item !== undefined) ? this.props.item.ConversationId : null;

            let reactParamsObjects = {
                data: data,
                conversationId: conversationId,
                dashboardGroupIdSelected: dashboardGroupId,
                isProjectTitle: this.props.isProjectTitle,
                hideImageProject: this.props.hideImageProject,
                hasOwnScope: this.props.config.hasOwnScope,
                configValueId: this.props.configValueId
            };

            let reactParamsFunctions = {
                fncToogleShowDetails: this.toogleShowDetails,
                fncReloadCustomFields: this.reloadCustomField,
                fncResetLayersToHide: this.resetLayersToHide,
                fncUpdateConversation: this.onUpdateConversation,
                fncChangeContainer: this.onChangeContainer,
                fncHide: this.hideDetails,
                fncShow: this.showDetails,
                fncRecoverFieldsValues: this.onRecoverFieldsValues,
                fncChangedItemType: this.onChangedItemType,
                fncUpdateLayersConversation: this.onUpdateLayersConversation,
                fncUpdateDetails: this.onUpdateDetails
            };

            this.props.config.onLoadDataCallback(this.refs.customFieldsContainer, reactParamsObjects, reactParamsFunctions);
        }
    }

    reloadCustomField(resetRequestState) {
        //We wait that props will be updated
        setTimeout(function () {
            if (resetRequestState) {
                let requestConfigCopy = Object.assign({}, this.state.requestConfig);
                requestConfigCopy.projectId = this.props.projectId;
                requestConfigCopy.configValueId = this.props.configValueId;
                requestConfigCopy.coreTaskId = (this.props.item !== undefined) ? this.props.item.CoreTaskId : null;
                requestConfigCopy.personId = (this.props.item !== undefined) ? this.props.item.PersonId : null;

                this.setState({ requestConfig: requestConfigCopy });
            }

            //We wait that state will be updated
            setTimeout(function () {
                const { config } = this.props;
                this.getCustomFieldsData(config, true);
            }.bind(this));
        }.bind(this));
    }

    UpdateCustomFieldsByLayer(layerCustomFieldsUpdated) {
        this.setState((prevState) => {
            let indexLayer = prevState.customFieldsbyLayer.findIndex((layerItem) => { return layerItem.LayerId === layerCustomFieldsUpdated.LayerId; });

            if (indexLayer >= 0) {
                prevState.customFieldsbyLayer[indexLayer] = layerCustomFieldsUpdated;
            }

            return prevState;
        });

        setTimeout(function () {
            if (this.props.config.UpdateDetailsConversationReact) {
                let conversationValidated = Object.assign({}, this.props.item);
                if (conversationValidated.ConfigValueId === 9) {
                    conversationValidated.ConversationGroups = this.state.groups;
                }

                this.props.config.UpdateDetailsConversationReact(this.state.customFieldsbyLayer, conversationValidated);
            }
        }.bind(this));
    }

    toogleShowDetails(callApi = true) {
        let showDetails = !this.state.showDetails;
        const { item, isProjectView } = this.props;

        if (isProjectView) {
            showDetails = true;

            var containerElement = $('#canvasDetails');
            if (containerElement.length > 0 && !containerElement[0].hasAttribute("data-is-react")) {
                containerElement.attr("data-is-react", true);

                this.setState({ containerSelector: "#canvasDetails" });
            }
        } else {
            var containerElement = $('li[data-conversationid=' + item.ConversationId + '] .show-details-panel');
            if (containerElement.length > 0 && !containerElement[0].hasAttribute("data-is-react") && showDetails) {
                containerElement.attr("data-is-react", true);

                this.setState({ containerSelector: "li[data-conversationid='" + item.ConversationId + "'] .show-details-panel" });
            }
            if (callApi) {
                this.setState({ showDetails: showDetails });
            }
        }

        if (showDetails) {
            setTimeout(function () {
                let config = {
                    callApi: callApi
                };

                this.getCustomFieldsData(config, true);
            }.bind(this));
        }
    }
    hideDetails() {
        this.setState({ showDetails: false });
    }

    showDetails() {
        var containerElement = $('li[data-conversationid=' + this.props.item.ConversationId + '] .show-details-panel');
        if (containerElement.length > 0 && !containerElement[0].hasAttribute("data-is-react")) {
            containerElement.attr("data-is-react", true);

            this.setState({ containerSelector: "li[data-conversationid='" + this.props.item.ConversationId + "'] .show-details-panel" });
        }
        this.setState({ showDetails: true });
    }

    render() {
        const {
            disableAutoSave,
            canEdit,
            customFieldsbyLayer,
            requestConfig,
            EditorEnabled,
            showDetails,
            layersToHide,
            isNew,
            containerSelector,
            showHeader,
            groups,
            highlight
        } = this.state;

        const { item, projectId, isProjectTitle, hideImageProject, isConversationDetailsView, config } = this.props;

        if (item === undefined) {
            return null;
        }

        const onCustomFieldUpdated = this.props.config.onCustomFieldUpdated ? this.props.config.onCustomFieldUpdated : null;

        function IsValidLayer(customFieldLayer, layersArrayToHide) {
            let returnValue = false;

            if (customFieldLayer.IsEnable === true && customFieldLayer.CustomFields && customFieldLayer.CustomFields.length > 0) {

                if (layersArrayToHide !== undefined && layersArrayToHide !== null && layersArrayToHide !== false && layersArrayToHide.length > 0) {
                    let layerFound = layersArrayToHide.findIndex(function (elem) { return elem === customFieldLayer.LayerId });
                    if (layerFound !== -1) {
                        returnValue = false;
                    } else {
                        returnValue = true;
                    }
                } else {
                    returnValue = true;
                }
            } else {
                returnValue = false;
            }
            return returnValue;
        }

        let showHeaderValidated = showHeader;
        if (isProjectTitle) {
            showHeaderValidated = false;
        }

        if (containerSelector && showDetails === true) {
            return (
                <RenderInBody>
                        <div data-is-react="true"
                            ref="customFieldsContainer"
                            className={(showDetails ? "corgee_com_custom-fields-container details-tab-animation-content layer-details-container react-item-highlighted" : "show-details-panel") + (highlight ? " react-item-highlighted" :"")}>
                            {
                                showDetails && showHeaderValidated &&
                                <HeaderDetails {...this.props}
                                    isNew={isNew}
                                    disableAutoSave={disableAutoSave}
                                    sendDataToCallback={this.sendDataToCallback}
                                    onUpdateConversationGroups={this.onUpdateConversation} />
                            }
                            {
                                showDetails ?
                                    (customFieldsbyLayer &&
                                        customFieldsbyLayer.map(customField =>
                                            customField !== undefined && customField.IsEnable && IsValidLayer(customField, layersToHide) === true ?
                                                <LayerItem key={"l-cf-" + customField.LayerId}
                                                    disableAutoSave={disableAutoSave}
                                                    canEdit={canEdit}
                                                    customField={customField}
                                                    configValueId={requestConfig.configValueId}
                                                    projectId={projectId}
                                                    apiConfig={requestConfig.api}
                                                    EditorEnabled={EditorEnabled}
                                                    UpdateCustomFieldsByLayer={this.UpdateCustomFieldsByLayer}
                                                    groups={groups}
                                                    isProjectTitle={isProjectTitle}
                                                    hideImageProject={hideImageProject}
                                                    onCustomFieldUpdated={onCustomFieldUpdated}
                                                    SocketConnectionId={config.SocketConnectionId} />
                                                : null
                                        ))
                                    : null
                            }
                        </div>
                    </ RenderInBody>
            );
        } else {
            return (
                <div data-is-react="true" ref="customFieldsContainer"
                    className={(showDetails ? "corgee_com_custom-fields-container details-tab-animation-content layer-details-container" : "show-details-panel") + (highlight ? " react-item-highlighted" : "")}>
                    {
                        showDetails && showHeaderValidated &&
                        <HeaderDetails {...this.props}
                            isNew={isNew}
                            disableAutoSave={disableAutoSave}
                            ref="convHeaderDetails"
                            sendDataToCallback={this.sendDataToCallback}
                            onUpdateConversationGroups={this.onUpdateConversation} />
                    }
                    {
                        showDetails ?
                            (customFieldsbyLayer &&
                                customFieldsbyLayer.map(customField =>
                                    customField !== undefined && customField.IsEnable && IsValidLayer(customField, layersToHide) === true ?
                                        <LayerItem key={"l-cf-" + customField.LayerId}
                                            disableAutoSave={disableAutoSave}
                                            canEdit={canEdit}
                                            customField={customField}
                                            configValueId={requestConfig.configValueId}
                                            projectId={projectId}
                                            apiConfig={requestConfig.api}
                                            EditorEnabled={EditorEnabled}
                                            UpdateCustomFieldsByLayer={this.UpdateCustomFieldsByLayer}
                                            groups={groups}
                                            isProjectTitle={isProjectTitle}
                                            hideImageProject={hideImageProject}
                                            onCustomFieldUpdated={onCustomFieldUpdated}
                                            isConversationDetailsView={isConversationDetailsView}
                                            SocketConnectionId={config.SocketConnectionId} />
                                        : null
                                ))
                            : null
                    }
                </div>
            );
        }
    }
}

CustomFields.defaultProps = {
    config: {},
    item: {},
    configValueId: null,
    projectId: null,
    groups: null,
    section: '',
    isProjectView: false
};

CustomFields.propTypes = {
    config: PropTypes.object,
    item: PropTypes.object.isRequired,
    configValueId: PropTypes.number,
    projectId: PropTypes.number,
    groups: PropTypes.array,
    section: PropTypes.string,
    isProjectTitle: PropTypes.bool,
    hideImageProject: PropTypes.bool,
    containerSelector: PropTypes.string,
    isProjectView: PropTypes.bool,
    isConversationDetailsView: PropTypes.bool,
    SocketConnectionId: PropTypes.string
}

export default CustomFields;

const LayerItem = ({
    disableAutoSave,
    canEdit,
    customField,
    configValueId,
    projectId,
    apiConfig,
    EditorEnabled,
    UpdateCustomFieldsByLayer,
    groups,
    isProjectTitle,
    hideImageProject,
    onCustomFieldUpdated,
    isConversationDetailsView,
    SocketConnectionId
}) => {

    let data = [];
    let copyCustomField = Object.assign({}, customField);
    let y = 0;

    if (isProjectTitle) {
        customField.CustomFields.map(function (item) {
            if (item.FieldTypeId === 6 || item.FieldTypeId === 7) {
                data[y] = item;
                y++;
            }
        });
        copyCustomField.CustomFields = data;
    }

    if (hideImageProject && customField.IsDefault) {
        y = 0;
        customField.CustomFields.map(function (item) {
            if (item.Name !== "Photo") {
                data[y] = item;
                y++;
            }
        });

        copyCustomField.CustomFields = data;
    }

    if (copyCustomField.CustomFields.length > 0) {
        return (
            <div className="corgee_com_custom-fields-color-wrapper _DEL_color-wrapper">
                <div className="corgee_com_custom-field-top-border _DEL_top-border enable-help-tooltip layer-details-tooltip"
                    data-title={customField.Name} style={{ borderColor: customField.Color }}></div>
                <LayersDetails disableAutoSave={disableAutoSave}
                    canEdit={canEdit}
                    customField={copyCustomField}
                    apiConfig={apiConfig}
                    EditorEnabled={EditorEnabled}
                    configValueId={configValueId}
                    projectId={projectId}
                    UpdateCustomFieldsByLayer={UpdateCustomFieldsByLayer}
                    groups={groups}
                    onCustomFieldUpdated={onCustomFieldUpdated}
                    isConversationDetailsView={isConversationDetailsView}
                    SocketConnectionId={SocketConnectionId} >
                </LayersDetails>
            </div>
        )
    } else {
        return (null);
    }
};

LayerItem.propTypes = {
    disableAutoSave: PropTypes.bool,
    canEdit: PropTypes.bool,
    customField: PropTypes.object,
    configValueId: PropTypes.number.isRequired,
    projectId: PropTypes.number,
    apiConfig: PropTypes.object,
    EditorEnabled: PropTypes.bool,
    UpdateCustomFieldsByLayer: PropTypes.func,
    isProjectTitle: PropTypes.bool,
    hideImageProject: PropTypes.bool,
    isConversationDetailsView: PropTypes.bool,
    SocketConnectionId: PropTypes.string,
    getForceReloadDetailsFn: PropTypes.func
};
