import React from 'react';
import PropTypes from 'prop-types';
import CorgeeStorageHelper from './globals/CorgeeStorageHelper';
import ConversationLayers from './globals/ConversationLayers';
import ConversationProjects from './globals/ConversationProjects';
import ConversationSecurityGroups from './globals/ConversationSecurityGroups';
import ConversationGroupLayers from './globals/ConversationGroupLayers';
import * as apiHelper from './api/Utils';

import './HeaderDetails.css';

class HeaderDetails extends React.PureComponent {
    constructor(props, context) {
        super(props, context);

        this.state = {
            apiConfig: {
                url: props.config.apiUrl,
                token: props.config.authToken
            },
            conversation: props.item,
            projectNameConfig: CorgeeStorageHelper.getProjectNameConfig(),
            isProject: (props.item && (props.item.ConfigValueId === 9 || props.item.ConfigValueID===9 || props.configValueId==9 )) ? true : false,
            isPeople: (props.item && props.item.ConfigValueId === 15) ? true : false,   
            isProjectView: (window.location.href.indexOf("ProjectDetails") > -1) ? true : false,
            isConversationDetails: (window.location.href.indexOf("ConversationDetails") > -1) ? true : false,
            personId: CorgeeStorageHelper.personId()
        }

        this.onChangedSecurityGroups = this.onChangedSecurityGroups.bind(this);
        this.onChangedGroupLayers = this.onChangedGroupLayers.bind(this);
        this.onChangedProject = this.onChangedProject.bind(this);
        this.onChangedLayers = this.onChangedLayers.bind(this);
        this.updateConversation = this.updateConversation.bind(this);
        this.reloadLayers = this.reloadLayers.bind(this);
        this.onChangedItemType = this.onChangedItemType.bind(this);
        this.addLayerToCard = this.addLayerToCard.bind(this);
        this.onUpdateLayersConversation = this.onUpdateLayersConversation.bind(this);
    }
    onUpdateLayersConversation(conversationLayers){
        let conversationCopy = Object.assign({}, this.state.conversation);
        conversationCopy.ConversationLayers = conversationLayers;

        this.setState({ conversation: conversationCopy});

        if(this.refs.ConversationLayersElement){
            this.refs.ConversationLayersElement.onSelectLayer(conversationLayers);
            this.refs.ConversationLayersElement.onUpdateOriginalSelectedIds(conversationLayers);
        }

        if(this.refs.ConversationGroupLayersElement){
            this.refs.ConversationGroupLayersElement.onSelectGroupLayers(conversationLayers);
        }
    }
    addLayerToCard(e, layer){        
        if (this.props.config.addLayerToCard && !this.props.isConversationDetailsView) {
            
            let conversationCopy0 = Object.assign({}, this.state.conversation);
            if (conversationCopy0.ConfigValueId===undefined){
                conversationCopy0.ConfigValueId = conversationCopy0.selectedConfigValueId;
            }

            this.props.config.addLayerToCard(e, conversationCopy0, layer, function (layerData){
                if (this.props.isNew === true) {
                    let newConversationCopy = Object.assign({}, this.state.conversation);
                    let newLayersSelected = newConversationCopy.ConversationLayers.filter(function(item){ 
                        return !item.isForAdd===true;
                    });
                    newLayersSelected.push(layerData);
                    newConversationCopy.ConversationLayers = newLayersSelected;
                    newConversationCopy.isAddLayerToCard = true;

                    this.setState({ conversation: newConversationCopy });

                    if (this.props.config.onChangeLayers) {
                        this.props.config.onChangeLayers(newConversationCopy);
                    }
                }
            }.bind(this));
        }        
        if (this.props.config.addLayerToCard && this.props.isConversationDetailsView) {
            
            this.props.config.addLayerToCard(e, layer, null);
        }        
    }
    onChangedItemType(newItemType){
        let isProjectCopy = false;
        if (newItemType.ConfigValueId === 9) {
            isProjectCopy = true;

            apiHelper.get(this.state.apiConfig, "api/group/GetGroups").then((dataResponse) => {

                let groupsResponse = dataResponse;
                apiHelper.get(this.state.apiConfig, "api/Person/PersonGroupProject?PersonId=" + this.state.personId).then((dataResponse) => {
                    if (groupsResponse.length > 0) {

                        let groupDefault = groupsResponse.find(function (item) { return item.GroupId==dataResponse; }); 
                        if(!groupDefault || groupDefault == null){
                            groupDefault = groupsResponse.find(function (item) { return item.IsDefault===true; });
                        }

                        let conversationCopy = Object.assign({}, this.state.conversation);
                        conversationCopy.ConversationGroups = [groupDefault];

                        this.setState({ conversation: conversationCopy });

                        if (this.props.onUpdateConversationGroups) {                        
                            this.props.onUpdateConversationGroups(conversationCopy);
                        }
                                                
                        if (this.refs.ConversationSecurityGroupsElement) {
                            this.refs.ConversationSecurityGroupsElement.setDefaultGroup(groupDefault);
                        }
                        this.props.sendDataToCallback(conversationCopy.layerCustomFields);
                    }
                }).catch((error) => {
                    console.warn("Api Error", error);
                });  
            }).catch((error) => {
                console.warn("Api Error", error);
            });  
    }
        this.setState({ isProject: isProjectCopy });        
    }
    componentWillReceiveProps(nextProps, nextState) { 
        if (this.state.conversation && nextProps.item 
            && this.state.conversation.ConfigValueId === 9
            && this.state.conversation.ConversationGroups!==undefined
            && this.state.conversation.ConversationGroups!==null
            && nextProps.item.ConversationGroups !== undefined
            && nextProps.item.ConversationGroups !== null
            && nextProps.item.ConversationGroups.length > 0
            && (this.state.conversation.ConversationGroups.length !== nextProps.item.ConversationGroups.length ||
                this.state.conversation.ConversationLayers.length !== nextProps.item.ConversationLayers.length)
            && (this.state.isProjectView === true || this.state.isConversationDetails===true)) 
        {
            let convCopy = Object.assign({}, this.state.conversation);
            convCopy.ConversationGroups = nextProps.item.ConversationGroups;
            convCopy.ConversationLayers = nextProps.item.ConversationLayers;
            
            this.setState({ conversation: convCopy });           

            if (this.refs.ConversationGroupLayersElement && nextProps.item.ConversationLayers) {
                this.refs.ConversationGroupLayersElement.onChangeParentLayers(nextProps.item.ConversationLayers);
            }
        }

        if (this.state.conversation && nextProps.item 
            && (this.state.conversation.ConfigValueId !== nextProps.item.ConfigValueId || this.state.conversation.selectedConfigValueId !== nextProps.item.selectedConfigValueId)             
        ) {

            let convCopy = Object.assign({}, this.state.conversation);
            convCopy.ConfigValueId = (nextProps.item.selectedConfigValueId) ? nextProps.item.selectedConfigValueId : nextProps.item.ConfigValueId;            
            convCopy.selectedConfigValueId = (nextProps.item.selectedConfigValueId) ? nextProps.item.selectedConfigValueId : nextProps.item.ConfigValueId;            

            debugger
            
            let isProjectCopy = false;
            if (convCopy.ConfigValueId===9){
                isProjectCopy = true;
            }
            this.setState({ conversation: convCopy, isProject: isProjectCopy});
        }
    }
    componentDidMount() {
        if (this.state.isProject 
            && this.state.conversation.ConfigValueId === 9 
            && (this.state.conversation.ConversationGroups===null || this.state.conversation.ConversationGroups.length===0)
        ){
            apiHelper.get(this.state.apiConfig, "api/group/GetGroupsByProject?projectId=" + this.state.conversation.ProjectId).then((dataResponse) => {
                if (dataResponse.length > 0) {
                    
                    let conversationCopy = Object.assign({}, this.state.conversation);
                    conversationCopy.ConversationGroups = dataResponse;

                    this.setState({conversation: conversationCopy});

                    if (this.props.onUpdateConversationGroups) {                        
                        this.props.onUpdateConversationGroups(conversationCopy);
                    }
                }
            }).catch((error) => {
                console.warn("Api Error", error);
            });;
        }

        if(this.state.isProject && this.props.isNew){

            let newItemType = {
                ConfigValueId: 9,
                DashboardGroupId: this.state.conversation.dashboardGroupId
            };
            this.onChangedItemType(newItemType);

        }

        if (this.state.conversation
            && this.state.conversation.ConfigValueId !== 9
            && (this.props.isNew === true)
        ) {
            this.reloadLayers();
        }
    }
    reloadLayers(stopPropagate){

        let params = "?";
        if (this.state.conversation.selectedConfigValueId) {
            params += "configValueId=" + this.state.conversation.selectedConfigValueId;
        }
        if (this.state.conversation.ProjectID || this.state.conversation.ProjectId) {
            if (this.state.conversation.ProjectID){
                params += "&projectId=" + this.state.conversation.ProjectID;
            }
            if (this.state.conversation.ProjectId) {
                params += "&projectId=" + this.state.conversation.ProjectId;
            }
        }

        var userDefaultLayerId = CorgeeStorageHelper.layerId();

        let conversationCopyBackup = Object.assign({}, this.state.conversation);

        /*
        if (this.props.config.onSetBackupCustomFields) {
            let conversationCopyBackup = Object.assign({}, this.state.conversation);
            this.props.config.onSetBackupCustomFields(conversationCopyBackup);
        }*/

        apiHelper.get(this.state.apiConfig, "api/layer/GetLayersByConfigValueId" + params).then((layersResponse) => {

            //dashboardGroupData.OriginalItem.layers = layers;

            let conversationCopy = Object.assign({}, this.state.conversation);
            conversationCopy.layers = layersResponse;

            let selectedLayer = layersResponse.find((item) => {
                return item.LayerId === parseInt(userDefaultLayerId);
            });

            if (selectedLayer) {
                if (CorgeeStorageHelper.isLayerHidden(selectedLayer.LayerId)) selectedLayer = undefined;
            }

            if (!selectedLayer) {
                let userLayers = CorgeeStorageHelper.getAllLayers();
                for (let i = 0; i < userLayers.length; i++) {
                    for (let j = 0; j < layersResponse.length; j++) {
                        if (userLayers[i].LayerId == layersResponse[j].LayerId && !CorgeeStorageHelper.isLayerHidden(layersResponse[j].LayerId)) {
                            selectedLayer = layersResponse[j];
                            break;
                        }
                        if (selectedLayer) break;
                    }
                }
            }

            if (selectedLayer) {
                conversationCopy.ConversationLayers = [selectedLayer];

                /*
                if (this.props.config.onChangeLayers && stopPropagate!==true) {
                    this.props.config.onChangeLayers(conversationCopy);
                }*/
                if (this.props.config.onChangedProject) {
                    this.props.config.onChangedProject(undefined, conversationCopy);
                }                
            }
            else{
                conversationCopy.ConversationLayers = [];
                if (this.props.config.onChangedProject) {
                    this.props.config.onChangedProject(undefined, conversationCopy);
                }                
            }      
            /*setTimeout(function(){
                if (this.props.config.onRecoverFieldsValues) {
                    this.props.config.onRecoverFieldsValues(conversationCopy);
                }
            }.bind(this));   */

            this.setState({ conversation: conversationCopy });            
        }).catch((error) => {
            console.warn("Api Error", error);
        });
    }
    updateConversation(conv){        
        this.setState({ conversation: conv});
    }
    onChangedSecurityGroups(e, params){        
        if (this.props.isNew === true) {

            let convCopy = Object.assign({}, this.state.conversation);
            convCopy.ConversationGroups = params.ConversationGroups;
            convCopy.ConversationLayers = params.ConversationLayers;
            
            this.setState({ conversation: convCopy });           
            if (this.props.onUpdateConversationGroups) {                        
                this.props.onUpdateConversationGroups(convCopy);
            }

            if (this.refs.ConversationGroupLayersElement){
                this.refs.ConversationGroupLayersElement.onChangeGroupsLayers(params.ConversationGroups);
            }
        } else {
            if (this.props.config.onChangedSecurityGroups) {
                this.props.config.onChangedSecurityGroups(e, params);
            }
        }
    }
    onChangedGroupLayers(e, params) {        
        if (this.props.config.onChangedGroupLayers) {
            this.props.config.onChangedGroupLayers(e, params);
        }
    }
    onChangedProject(e, params) {                 
        this.setState({ conversation: params })
        
        if (this.props.isNew === true){   
            setTimeout(function(){
                this.reloadLayers(true);
            }.bind(this));
        } else {
            if (this.props.config.onChangedProject) {
                this.props.config.onChangedProject(e, params);
            }
        }
    }
    onChangedLayers(params, isFromInfoWindow) {        
        this.setState({ conversation: params})

        if(this.props.config.onChangeLayers){
            this.props.config.onChangeLayers(params, isFromInfoWindow);
        }
    }
    render() {
        const { apiConfig, conversation, projectNameConfig, isProject, isPeople, isProjectView } = this.state;
        const { isNew, disableAutoSave, section } = this.props;        

        return (
            <div className="corgee_com_custom-fields-color-wrapper _DEL_color-wrapper no-layer">
                <ul className="_DEL_inner">
                    {
                        isProject === true && isPeople === false &&
                        <li className="corgee_com_custom-fields-field-A _DEL_field-A group-editable-field-item">
                            <span className="corgee_com_custom-fields-title _DEL_title">Security Groups</span>
                            <ConversationSecurityGroups apiConfig={apiConfig} conversation={conversation} ref="ConversationSecurityGroupsElement"
                                isNew={isNew} disableAutoSave={disableAutoSave} 
                                onChangedSecurityGroups={this.onChangedSecurityGroups} />
                        </li>
                    }
                    {
                        isProject === true && isPeople === false &&
                        <li className="corgee_com_custom-fields-field-A _DEL_field-A group-layer-editable-field-item">
                            <span className="corgee_com_custom-fields-title _DEL_title">Layers</span>
                            <ConversationGroupLayers apiConfig={apiConfig} conversation={conversation} ref="ConversationGroupLayersElement"
                                isNew={isNew} disableAutoSave={disableAutoSave}
                                onChangedGroupLayers={this.onChangedGroupLayers} />
                        </li>
                    }
                    {
                        isProject === false && isPeople === false && isProjectView===false &&
                        <li className="corgee_com_custom-fields-field-A _DEL_field-A project-editable-field-item">
                            <span className="corgee_com_custom-fields-title _DEL_title">{projectNameConfig}</span>
                            <ConversationProjects apiConfig={apiConfig} conversation={conversation}
                                isNew={isNew} disableAutoSave={disableAutoSave} onChangedProject={this.onChangedProject} />
                        </li>
                    }
                    {
                        isProject === false && isPeople === false &&
                        <li className="corgee_com_custom-fields-field-A _DEL_field-A layer-editable-field-item">
                            <span className="corgee_com_custom-fields-title _DEL_title">Layers</span>                            
                            <ConversationLayers isNew={isNew} apiConfig={apiConfig} conversation={conversation} ref="ConversationLayersElement"
                                isNew={isNew} disableAutoSave={disableAutoSave} onChangedLayers={this.onChangedLayers} addLayerToCard={this.addLayerToCard} section={section} />
                        </li>
                    }
                </ul>
            </div>
        );
    }
}

HeaderDetails.propTypes = {
    config: PropTypes.object.isRequired,    
    item: PropTypes.object.isRequired,
    disableAutoSave: PropTypes.bool,
    isNew: PropTypes.bool,    
};

HeaderDetails.defaultProps = {
    disableAutoSave: false,
    isNew: false
};

export default HeaderDetails;
