import React from 'react';
import PropTypes from 'prop-types';
import HyperLinkField from './fields/HyperLinkField';
import NumericIntegerField from './fields/NumericIntegerField';
import InputTextField from './fields/InputTextField';
import NumericDecimalField from './fields/NumericDecimalField';
import ComboBoxField from './fields/ComboBoxField';
import InputMultiTextField from './fields/InputMultiTextField';
import InputDateField from './fields/InputDateField';
import MultiSelectField from './fields/MultiSelectField';
import ImageField from './fields/ImageField';

const CustomFieldItem = ({
    disableAutoSave,
    canEdit,
    customField,
    EditorEnabled,
    apiConfig,
    configValueId,
    projectId,
    UpdateCustomFields,
    groups,
    onCustomFieldUpdated,
    isConversationDetailsView,
    SocketConnectionId
}) => {

    let fieldType = customField.FieldTypeId ? customField.FieldTypeId : customField.FieldType.FieldTypeId;

    const makeCustomField = (fieldType,
        disableAutoSave,
        canEdit,
        customField,
        EditorEnabled,
        apiConfig,
        configValueId,
        projectId,
        UpdateCustomFields,
        groups,
        onCustomFieldUpdated,
        SocketConnectionId) => {

        customField.SocketConnectionId = SocketConnectionId;
        switch (fieldType) {
            case 1:
                return (
                    <InputTextField key={"cfs-" + customField.CustomFieldId}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}>
                    </InputTextField>
                );

            case 2:
                return (
                    <NumericIntegerField key={"cfs-" + customField.CustomFieldId}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}>
                    </NumericIntegerField>
                );

            case 3:
                return (
                    <NumericDecimalField key={"cfs-" + customField.CustomFieldId}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}>
                    </NumericDecimalField>
                )

            case 4:
                return (
                    <NumericIntegerField key={"cfs-" + customField.CustomFieldId}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}>
                    </NumericIntegerField>
                );

            case 5:
                return (
                    <InputMultiTextField key={"cfs-" + customField.CustomFieldId}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}>
                    </InputMultiTextField>
                );

            case 6:
                return (
                    <InputDateField key={"cfs-" + customField.CustomFieldId}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}
                        onCustomFieldUpdated={onCustomFieldUpdated}>
                    </InputDateField>
                );

            case 9:
                return (
                    <ComboBoxField disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}
                        isConversationDetailsView={isConversationDetailsView} >
                    </ComboBoxField>
                );

            case 7:
                return (
                    <MultiSelectField configValueId={configValueId}
                        projectId={projectId}
                        groups={groups}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        UpdateCustomFields={UpdateCustomFields}
                        isConversationDetailsView={isConversationDetailsView} />
                );

            case 10:
                return (
                    <ImageField
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        UpdateCustomFields={UpdateCustomFields} />
                );

            case 11:
                return (
                    <HyperLinkField key={"cfs-" + customField.CustomFieldId}
                        disableAutoSave={disableAutoSave}
                        canEdit={canEdit}
                        customField={customField}
                        apiConfig={apiConfig}
                        EditorEnabled={EditorEnabled}
                        UpdateCustomFields={UpdateCustomFields}>
                    </HyperLinkField>
                );
        }
    }

    return (
        <li>
            {
                makeCustomField(fieldType,
                    disableAutoSave,
                    canEdit,
                    customField,
                    EditorEnabled,
                    apiConfig,
                    configValueId,
                    projectId,
                    UpdateCustomFields,
                    groups,
                    onCustomFieldUpdated,
                    SocketConnectionId)
            }
        </li>
    );
};

class LayersDetails extends React.Component {

    constructor(props) {
        super(props);
        this.state = {
            disableAutoSave: props.disableAutoSave,
            EditorEnabled: props.EditorEnabled,
            canEdit: props.canEdit,
            customFields: props.customField,
            apiConfig: props.apiConfig
        }
        this.UpdateCustomFields = this.UpdateCustomFields.bind(this);
    }

    componentWillReceiveProps(nextProps) {
        this.setState({ customFields: nextProps.customField });
    }

    UpdateCustomFields(updatedCustomField, e) {

        this.setState((customFields) => {
            for (var x in customFields.customFields.CustomFields) {
                if (customFields.customFields.CustomFields[x].CustomFieldId === updatedCustomField.CustomFieldId) {
                    customFields.customFields.CustomFields[x] = updatedCustomField;
                }
            };
            return customFields;
        });

        setTimeout(function () {
            if (this.props.UpdateCustomFieldsByLayer) {
                this.props.UpdateCustomFieldsByLayer(this.state.customFields);
            }
        }.bind(this));
    }

    render() {
        const { disableAutoSave, canEdit, customFields, EditorEnabled, apiConfig } = this.state;
        const { configValueId, projectId, groups, onCustomFieldUpdated, isConversationDetailsView, SocketConnectionId } = this.props;

        return (
            <ul className="_DEL_inner" data={this.UpdateCustomFields}>
                {
                    customFields.CustomFields.map(customField =>
                        <CustomFieldItem key={"li-" + customField.CustomFieldId}
                            disableAutoSave={disableAutoSave}
                            canEdit={canEdit}
                            customField={customField}
                            apiConfig={apiConfig}
                            EditorEnabled={EditorEnabled}
                            configValueId={configValueId}
                            projectId={projectId}
                            UpdateCustomFields={this.UpdateCustomFields}
                            groups={groups}
                            onCustomFieldUpdated={onCustomFieldUpdated}
                            isConversationDetailsView={isConversationDetailsView}
                            SocketConnectionId={SocketConnectionId} />
                    )
                }
            </ul>
        )
    }
}

CustomFieldItem.propTypes = {
    disableAutoSave: PropTypes.bool,
    canEdit: PropTypes.bool,
    customField: PropTypes.object,
    EditorEnabled: PropTypes.bool,
    apiConfig: PropTypes.object,
    UpdateCustomFields: PropTypes.func,
    groups: PropTypes.array,
    isConversationDetailsView: PropTypes.bool,
    SocketConnectionId: PropTypes.string
};

export default LayersDetails;

