import fetch from 'isomorphic-fetch';

export function get(apiConfig, apiControllerCall, requestPayload, ignoreParseJson) {
    return callFetch(apiConfig, apiControllerCall, "GET", requestPayload, ignoreParseJson);
}

export function post(apiConfig, apiControllerCall, requestPayload, ignoreParseJson) {
    return callFetch(apiConfig, apiControllerCall, "POST", requestPayload, ignoreParseJson);
}

export function put(apiConfig, apiControllerCall, requestPayload, ignoreParseJson) {
    return callFetch(apiConfig, apiControllerCall, "PUT", requestPayload, ignoreParseJson);
}

/** 
* Provide main functions for make call to any API
**/
export function callFetch(apiConfig, apiControllerCall, method, requestPayload, ignoreParseJson){
    let config = {        
        headers: { 
            'Accept': 'application/json',
            'Authorization': 'Bearer ' + apiConfig.token,
            'Content-Type': 'application/json;charset=UTF-8',
        },
    };
    if(method!==undefined && method!==null){
        config.method = method;
    }
    if(requestPayload!==undefined && requestPayload!==null){
        config.body = JSON.stringify(requestPayload);
    }    

    return new Promise((resolve, reject) => {
        fetch(apiConfig.url + apiControllerCall, config)        
        .then((response) => {       	            
            if(response.ok){        
                if(ignoreParseJson)  {                       
                    return response;   
                } else{
                    return response.json();
                }             
            } else {                
                throw Error(response.statusText);              
            }
        })
        .then(data => {
            resolve(data);
        })
        .catch(error => {            
            reject('There was an error.');            
        });
    });
}