import React from 'react';
var ReactDOM = require('react-dom');
import CustomFieldsService from '../api/CustomFieldsService';
import $ from 'jquery';
import Select from 'react-select';
import 'react-select/dist/react-select.css';
import * as domUtil from './DOMHelper';

import './ComboBoxField.css';

// import { RenderToBody } from "../globals/AppendToBody";
import RenderInBody from "../globals/RenderInBody";
import shortid from 'shortid';
import ClickOutSide from '../globals/ClickOutSide';
import * as CorgeeHelper from '../CorgeeHelper';

class ComboBoxField extends React.Component{

    constructor(props){
        super(props);
        this.state = {
            selectedValue: {
                label: props.customField.SelectedValuesText != "" ? props.customField.SelectedValuesText : "",
                value: props.customField.Value != null ? parseInt(props.customField.Value) : ""
            },
            EditorEnabled: props.EditorEnabled,
            editableValue: "",
            disableAutoSave: props.disableAutoSave,
            customField: props.customField,
            canEdit: props.canEdit && props.customField.ConversationIsEnable,
            apiConfig: props.apiConfig,
            previousValue: "",
            listMenuStyle: {                
                width: 150
            },
            scrollTop: 0,
            elementId: shortid.generate(),
            isProjectView: (window.location.href.indexOf("ProjectDetails") > -1) ? true : false,
            isFromInfoWindow: false,
            isFromTabDetails: false,
            isConversationDetailsView: (window.location.href.indexOf("ConversationDetails") > -1) ? true : false,

            listPosition: {
                left: 0,
                top: 0
            }
        }

        this.handleChange = this.handleChange.bind(this);
        this.save = this.save.bind(this);
        this.enableEditor = this.enableEditor.bind(this);
        this.onBlur = this.onBlur.bind(this);
        this.onUpdateCustomField = this.onUpdateCustomField.bind(this);
        this.loadData = this.loadData.bind(this);        
        this.onApplyStyle = this.onApplyStyle.bind(this);
        this.onCloseSelect = this.onCloseSelect.bind(this);
    }
    componentDidMount() {
        if (this.refs.comboBoxRef) {
            let containerElement = $(this.refs.comboBoxRef);
            if (containerElement.closest(".details-tab-directive").length > 0) {
                this.setState({ isFromInfoWindow: true });
            }   
            if(containerElement.closest('#canvasDetails').length>0){
                this.setState({ isFromTabDetails: true });
            }
        }
    }
    onApplyStyle() {

        if (!this.state.isFromInfoWindow && !this.state.isFromTabDetails) {
            let isProjectView = (window.location.href.indexOf("ProjectDetails") > -1) ? true : false;
            this.setState({ isProjectView: isProjectView });

            let containerElement = $(this.refs.comboBoxRef);

            let targetElement = containerElement.find("#select-target-" + this.state.elementId);        
            let targetTopValue = targetElement.offset().top;                        

            if (window.scrollY>0) {
                targetTopValue -= window.scrollY;
            }

            let selectElement = $("#select-render-" + this.state.elementId);
            selectElement.find('.Select').first().attr("style", "position:fixed; top: " + targetTopValue + "px; left: " + targetElement.offset().left + "px");
            
            containerElement.closest(".layer-custom-fields-container").first().on("scroll", this.onCloseSelect);
        }
    }
    onCloseSelect() {
        this.setState({ EditorEnabled: false });

        //let containerElement = $(this.refs.comboBoxRef);
        //containerElement.closest(".layer-custom-fields-container").first().off("scroll wheel DOMMouseScroll mousewheel", this.onCloseSelect);
    }

    componentWillReceiveProps(nextProps){
        let data = {
            value:nextProps.customField.Value != null ? parseInt(nextProps.customField.Value) : "",
            label:nextProps.customField.SelectedValuesText != "" ? nextProps.customField.SelectedValuesText : ""
        }
        this.setState({selectedValue: data});

        this.setState({customField:nextProps.customField, value:nextProps.customField.Value});
    }

    handleChange(event){        
        if(event != null) {            
            let data = {
                value: event[this.state.customField.ValueField],
                label: event[this.state.customField.TextField]
            }
            this.setState({ selectedValue: data }, this.onBlur);
        }
        else{
            var blank = {value:"", label:""};
            this.setState({ selectedValue: blank }, this.onBlur);
        }
    }

    save() {
        const {customField, selectedValue, editableValue, disableAutoSave, apiConfig} = this.state;

        var copyCustomField = Object.assign({}, this.state.customField);

        // We make sure that the new value is different to the previous one
        if (selectedValue.value !== editableValue && selectedValue.value != null ) {

            // Assign the new values
            copyCustomField.Value = selectedValue.value;
            copyCustomField.SelectedValuesText = selectedValue.label;

            this.setState({editableValue:selectedValue.value});
            
            //Do not save if autoSaved is disabled
            if (disableAutoSave == "false" || disableAutoSave === undefined || !disableAutoSave) {

                let requestSave = {
                    api: this.props.apiConfig,
                    request: copyCustomField
                }; 

                //this.disableEditor();

                if (copyCustomField.CustomFieldValueId === undefined || copyCustomField.CustomFieldValueId === null || copyCustomField.CustomFieldValueId === 0) {

                    CustomFieldsService.savePost(requestSave).then(function(data){
                        copyCustomField.CustomFieldValueId = data.CustomFieldValueId;
                                                
                        try{
                            this.setState({customField: copyCustomField});

                            setTimeout(function(){
                                this.onUpdateCustomField();
                            }.bind(this));
                        }
                        catch( err )
                        {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));

                } else {

                    CustomFieldsService.save(requestSave).then(function(data){
                        try{
                            this.setState({customField: copyCustomField});

                            setTimeout(function(){
                                this.onUpdateCustomField();
                            }.bind(this));
                        }
                        catch (err) {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));

                }
            }
            else {
                // Disable the control
                //this.disableEditor();
    
                this.setState({customField: copyCustomField});
                setTimeout(function(){
                    this.onUpdateCustomField();
                }.bind(this));
            }
         }

        this.setState({EditorEnabled:false});
    }    

    enableEditor(e){
        const {canEdit, customField, selectedValue, isConversationDetailsView} = this.state;
        if (canEdit) {
            
            if(!isConversationDetailsView && !CorgeeHelper.isSafari()){
                $('body').css('overflow', 'hidden');
            }

            setTimeout(function(){
                this.loadData();
                
                if (this.refs.selectComboBox) {
                    this.refs.selectComboBox.focus();
                }
            }.bind(this));
            
            
            let element = $(this.refs.comboBoxRef);
            let window = $(".conversation-editor");

            if(window.length > 0){
                element.closest(".layer-details-container").css('overflow', 'hidden');
            }
            
            var infoWindowScrollContainer = element.closest('.details-tab-directive .card-body-area');
            if (infoWindowScrollContainer.length > 0) {
                infoWindowScrollContainer.css('overflow', '');
            }        
    
            var initialScroll2 = element.closest(".sortableDashboard").scrollTop();
            element.closest(".sortableDashboard").on('scroll', function (e) {
                var newScroll2 = element.closest(".sortableDashboard").scrollTop();
                if (initialScroll2 != newScroll2) {
                    this.onBlur();
                }
            }.bind(this));
    
            var initialScroll3 = element.closest("md-dialog-content").scrollTop();
            element.closest("md-dialog-content").on('scroll', function (e) {
                var newScroll3 = element.closest("md-dialog-content").scrollTop();
                if (initialScroll3 != newScroll3) {
                    this.onBlur();
                }
            }.bind(this));
    
            setTimeout(function(){
                var initialScroll4 = element.closest(".card-body-area").scrollTop();
                element.closest(".card-body-area").on('scroll', function (e) {
                    var newScroll4 = element.closest(".card-body-area").scrollTop();
                    if (initialScroll4 != newScroll4) {
                        this.onBlur();
                    }
                }.bind(this));
            }.bind(this));
            
            var initialScroll5 = element.closest(".add-inline-task").scrollTop();
            element.closest(".add-inline-task").on('scroll', function (e) {
                var newScroll5 = element.closest(".add-inline-task").scrollTop();
                if (initialScroll5 != newScroll5) {
                    this.onBlur();
                }
            }.bind(this));
    
            element.closest('.layer-custom-fields-container').on('scroll', function (e) {
                this.onBlur();
            }.bind(this));        
        
            let elementPosition = e.currentTarget.getBoundingClientRect();

            this.setState({
                EditorEnabled:true, 
                editableValue:selectedValue.value,
                listPosition: {
                    left: elementPosition.x,
                    top: elementPosition.y
                }
            });
        }        
    }

    onBlur(){
        const {selectedValue, customField, previousValue} = this.state;
        
        $('body').css('overflow', '');

        var element = $(this.refs.comboBoxRef);
        element.closest(".layer-details-container").css('overflow', '');
        element.closest(".layer-details-container").unbind("scroll");
        element.closest(".card-body-area").unbind("scroll");
        element.closest("md-dialog-content").unbind("scroll");
        //element.closest(".sortableDashboard").unbind("scroll");
        element.closest(".add-inline-task").unbind("scroll");
        //element.closest('.layer-custom-fields-container').unbind('scroll');

        // Get new values
        if (selectedValue.value.length == 0) {
            
            //var copyCustomField = Object.assign({}, this.state.customField);

            this.setState({previousValue: selectedValue.Value});
            this.save();
            return;
        }

        if ((previousValue == selectedValue.value) || !selectedValue.label) {
            //this.disableEditor();
            this.onCloseSelect();
            return;
        }
               
        // Save changes
        this.save();                    
    }
    
    loadData() {
        const {customField} = this.state;
        if (customField) {
            
            let requestSave = {
                api: this.props.apiConfig,
                serviceUrl: customField.ServiceUrl  
            }; 

            CustomFieldsService.getListComboBox(requestSave).then(function(data){
                this.setState({options:data});
            }.bind(this));
        }
    }

    onUpdateCustomField(){
        if(this.props.UpdateCustomFields){
            this.props.UpdateCustomFields(this.state.customField);
        }
    }        
    render() {
        
        const { EditorEnabled, editableValue, canEdit, selectedValue, options, customField, listMenuStyle, isFromInfoWindow, isFromTabDetails, isConversationDetailsView, listPosition} = this.state;

        let isProjectView = (window.location.href.indexOf("ProjectDetails") > -1) ? true : false;
        let style = { position: 'fixed', zIndex: 100, top: listPosition.top, left: listPosition.left };

        // if (isProjectView){
        //     style.top = 15;
        // }

        let containerSelectorId = "select-target-" + this.state.elementId;
        let containerSelectorTarget = "#subtree-container";
        let isModalEditor = $(this.refs.comboBoxRef).closest('.conversation-editor').length > 0 ? true : false;

        if (isFromInfoWindow || isFromTabDetails || (isConversationDetailsView && !isModalEditor)) {
            containerSelectorTarget = "#" + containerSelectorId;
            style.fontSize = 15;
            style.position = "absolute";
        }

        return (
            <div ref="comboBoxRef" className="corgee_com_custom-fields-field-A _DEL_field-A custom-field-id" >
                <span className="corgee_com_custom-fields-title _DEL_title">{customField.Name}</span>

                <div className="click-to-edit">
                    {!EditorEnabled ? <div onClick={this.enableEditor} className={canEdit ? 'lbl-editable-input' : 'lbl-non-editable-input'}>
                        {selectedValue.label != undefined && selectedValue.label != "" ? selectedValue.label : "Not Set"}
                    </div> : null}

                    {EditorEnabled ? 
                        <div id={"select-target-" + this.state.elementId} style={{ margin: 0, padding: 0 }} >
                            {/* <RenderToBody subtreeContainer={containerSelectorTarget}> */}
                            <RenderInBody>
                                <div id={"select-render-" + this.state.elementId} className="corgee_com_custom-fields-combobox _DEL_comboBoxRef" style={style}>
                                    <ClickOutSide onClickOutSide={this.onBlur} /*whiteList={clickOutSideWhiteList}*/ >
                                    <Select name="form-field-name" ref="selectComboBox" 
                                        value={selectedValue.value}  
                                        labelKey={customField.TextField} 
                                        valueKey={customField.ValueField}
                                        options={options} 
                                        style={{ width: 150 }} 
                                        onChange={this.handleChange}
                                        onOpen={this.onApplyStyle}
                                        onBlur={this.onBlur}
                                        searchable={true} 
                                        openOnFocus={true} 
                                        autoFocus={true}
                                        optionClassName="Select-option-tether"
                                    ></Select>
                                    </ClickOutSide>
                                </div>
                            </ RenderInBody>
                            {/* </RenderToBody> */}
                        </div> : null}
                </div>
            </div>
            )

    }
}

export default ComboBoxField;