import React from 'react';
import CustomFieldsService from '../api/CustomFieldsService';
import $ from 'jquery';
import ClickOutSide from '../globals/ClickOutSide';

import './HyperLinkField.css';

class HyperLinkField extends React.Component{

    constructor(props){
        super(props);
        this.state = {
            value: props.customField.Value === null ? "" : props.customField.Value, 
            EditorEnabled: props.EditorEnabled,
            editableValue: "",
            disableAutoSave: props.disableAutoSave,
            customField: props.customField,
            canEdit: props.canEdit && props.customField.ConversationIsEnable,
            apiConfig: props.apiConfig
        }

        this.handleChange = this.handleChange.bind(this);
        this.save = this.save.bind(this);
        this.enableEditor = this.enableEditor.bind(this);
        this.disableEditor = this.disableEditor.bind(this);
        this.onUpdateCustomField = this.onUpdateCustomField.bind(this);
    }

    componentWillReceiveProps(nextProps){
        if ((nextProps.customField.Value !== this.state.customField.Value)
            || (nextProps.customField.CustomFieldValueId !== this.state.customField.CustomFieldValueId)) {            
            this.setState({ customField: nextProps.customField, value: nextProps.customField.Value === null ? "" : nextProps.customField.Value });
        }        
    }

    handleChange(event){
        this.setState({value: event.target.value});
    }

    save(event){
        const {disableAutoSave, editableValue, value, customField} = this.state;

        var copyCustomField = Object.assign({}, customField);

        if (value !== editableValue && value != null) {
            
            // Assign the new values
            copyCustomField.Value = value;

            this.setState({editableValue:value});
            /*this.setState( (customField) => {
                customField.Value = this.state.value;
                return customField;
            });*/

            this.disableEditor();

            if (value != "") {
                if ((value.substring(0, 7) != "http://") && (value.substring(0, 8) != "https://") && (value.substring(0, 6) != "ftp://")) {
                    copyCustomField.Value = "http://" + value;
                    this.setState({value:copyCustomField.Value, editableValue:copyCustomField.Value});
                }
            }

            //Do not save if autoSaved is disabled
            if (disableAutoSave == "false" || disableAutoSave === undefined || !disableAutoSave) {
                let requestSave = {
                    api: this.props.apiConfig,
                    request: copyCustomField
                }; 

                if (copyCustomField.CustomFieldValueId === undefined || copyCustomField.CustomFieldValueId === null || copyCustomField.CustomFieldValueId === 0) {

                    CustomFieldsService.savePost(requestSave).then(function(data){
                        copyCustomField.CustomFieldValueId = data.CustomFieldValueId;
                                                
                        try{
                            this.setState({ customField: copyCustomField }, () => {
                                this.onUpdateCustomField();
                            });                            
                        }
                        catch( err )
                        {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,1, element);
                    }.bind(this));

                } else {
                    CustomFieldsService.save(requestSave).then(function(data){
                        try{
                            this.setState({ customField: copyCustomField }, () => {
                                this.onUpdateCustomField();
                            });
                        }
                        catch (err) {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,1, element);
                    }.bind(this));
                }
            }
            else {
                // Disable the control
                this.disableEditor();
                
                this.setState({ customField: copyCustomField }, () => {
                    this.onUpdateCustomField();
                });
            }
        } else {
            // Disable the control
            this.disableEditor();
        }
    }

    enableEditor(){
        const {canEdit, value} = this.state;

        if (canEdit) {
            var element = $(this.refs.fieldA);
            var a = element.offset().left + 100;
            var b = element.closest('.layer-details-container').offset().left + element.closest('.layer-details-container').outerWidth();

            if (a > b) {
                element.find('input').css({
                    'width': '50px',
                });
            }

            this.setState({EditorEnabled:true, editableValue:value});
        }
    }

    disableEditor() {
        this.setState({EditorEnabled:false});
    };

    onUpdateCustomField(){
        if(this.props.UpdateCustomFields){
            this.props.UpdateCustomFields(this.state.customField);
        }
    }

    render() {
        const {EditorEnabled, canEdit, value, customField} = this.state;

        return (
            <div key={"cf-"+ customField.CustomFieldId} ref="fieldA" className="corgee_com_custom-fields-field-A _DEL_field-A custom-field-id">

                <span className="corgee_com_custom-fields-title _DEL_title">{customField.Name}</span>
                <i className="material-icons not-filterable enable-help-tooltip" data-title="This detail type is not filterable" >do_not_disturb_on</i>

                <div className="click-to-edit">
                    <div className="lbl-editable-hyperlink">
                        { !EditorEnabled && value != undefined && value != "" ? <div>
                            <a href={value} target="_blank" className="lbl-editable-link">{value}</a>

                            { canEdit ? <a onClick={this.enableEditor} className="enable-editor-button"><span className="icon-pencil-icon"></span></a> : null }

                        </div> : null }
                        { !EditorEnabled && (value == undefined || value == "") ? <div onClick={this.enableEditor} className={canEdit ? 'lbl-editable-input' : 'lbl-non-editable-input'}>Not set</div> : null }
                    </div>
                   
                    { EditorEnabled ? 
                        <ClickOutSide onClickOutSide={this.save} /*whiteList={clickOutSideWhiteList}*/ >
                            <div className="corgee_com_custom-fields-input-hyperlink">
                                <div className="ctrl-editable-input" style={{left:0,right:'initial'}}>
                                    <input value={value} onChange={this.handleChange} onBlur={this.save} aria-invalid="false" autoFocus/>
                                </div> 
                            </div>
                        </ClickOutSide>
                        : null }
                </div>
            </div>
        )
    }
}

export default HyperLinkField;

