import React from 'react';
import CustomFieldsService from '../api/CustomFieldsService';
import $ from 'jquery';
import RichTextEditor from './RichTextEditor';
import { Popover, Overlay } from 'react-bootstrap';
import { findDOMNode } from 'react-dom';

import '../ComponentHelpers.css';
import './InputMultiTextField.css';
import ClickOutSide from '../globals/ClickOutSide';

class InputMultiTextField extends React.Component {

    constructor(props) {
        super(props);
        this.state = {
            value: "",
            body: props.customField.Value === null ? "" : props.customField.Value,
            EditorEnabled: props.EditorEnabled,
            editableValue: "",
            disableAutoSave: props.disableAutoSave,
            customField: props.customField,
            canEdit: props.canEdit && props.customField.ConversationIsEnable,
            apiConfig: props.apiConfig,
            showPopup: false,
            popUpOpen: false,
            popUpPosition: "left",
            isItemDetails: (window.location.href.indexOf("ConversationDetails") > -1) ? true : false,
            isFromInfoWindow: false
        }

        this.handleClose = this.handleClose.bind(this);
        this.save = this.save.bind(this);
        this.enableEditor = this.enableEditor.bind(this);
        this.disableEditor = this.disableEditor.bind(this);
        this.onChangeSummernote = this.onChangeSummernote.bind(this);
        this.handleClick = this.handleClick.bind(this);
        this.onUpdateCustomField = this.onUpdateCustomField.bind(this);
        this.validatePosition = this.validatePosition.bind(this);
    }

    componentDidMount(){

        if (this.refs.popoverRef){
            let containerElement = $(this.refs.popoverRef);
            if (containerElement.closest(".details-tab-directive").length > 0) {
                this.setState({isFromInfoWindow: true});
            }
        }
    }
    componentWillReceiveProps(nextProps) {
        this.setState({ customField: nextProps.customField, body: nextProps.customField.Value === null ? "" : nextProps.customField.Value });
    }

    handleClose(e) {
        this.setState({ popUpOpen: false });
    }

    onChangeSummernote(content) {
        this.setState({ value: content });
    }

    handleClick() {
        this.setState({ showPopup: false });
        this.disableEditor();
    }

    save() {
        const { customField, value, body, editableValue, disableAutoSave, apiConfig } = this.state;

        var copyCustomField = Object.assign({}, this.state.customField);

        // We make sure that the new value is different to the previous one
        if (value !== editableValue && value != null) {

            // Assign the new values
            copyCustomField.Value = value;
            this.setState({ editableValue: value, body: value });

            this.disableEditor();

            //Do not save if autoSaved is disabled
            if (disableAutoSave == "false" || disableAutoSave === undefined || !disableAutoSave) {

                let requestSave = {
                    api: this.props.apiConfig,
                    request: copyCustomField
                };

                if (copyCustomField.CustomFieldValueId === undefined || copyCustomField.CustomFieldValueId === null || copyCustomField.CustomFieldValueId === 0) {

                    CustomFieldsService.savePost(requestSave).then(function (data) {
                        copyCustomField.CustomFieldValueId = data.CustomFieldValueId;

                        try {
                            this.setState({ customField: copyCustomField });

                            setTimeout(function () {
                                this.onUpdateCustomField();
                            }.bind(this));
                        }
                        catch (err) {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));

                } else {

                    CustomFieldsService.save(requestSave).then(function (data) {
                        try {
                            this.setState({ customField: copyCustomField });

                            setTimeout(function () {
                                this.onUpdateCustomField();
                            }.bind(this));
                        }
                        catch (err) {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));

                }
            }
            else {
                this.setState({ customField: copyCustomField });
                setTimeout(function () {
                    this.onUpdateCustomField();
                }.bind(this));
            }
        } else {
            // Disable the control
            this.disableEditor();
        }
    }

    enableEditor() {
        const { canEdit, value } = this.state;
        if (canEdit) {
            this.validatePosition();

            this.setState({ EditorEnabled: true, editableValue: value, showPopup: true, popUpOpen: !this.state.popUpOpen });

            $('.corgee_com_text-editor-container').remove();
            $('body').find('.note-popover').attr("style", 'display:none');

            var element = $(this.refs.popoverRef);
            element.closest(".layer-details-container").css('overflow', 'hidden');

            var infoWindowScrollContainer = element.closest('.details-tab-directive .card-body-area');
            if (infoWindowScrollContainer.length > 0) {
                infoWindowScrollContainer.css('overflow', '');
            }

            var initialScroll2 = element.closest(".sortableDashboard").scrollTop();
            element.closest(".sortableDashboard").on('scroll', function (e) {
                var newScroll2 = element.closest(".sortableDashboard").scrollTop();
                if (initialScroll2 != newScroll2) {
                    this.disableEditor();
                }
            }.bind(this));

            var initialScroll3 = element.closest("md-dialog-content").scrollTop();
            element.closest("md-dialog-content").on('scroll', function (e) {
                var newScroll3 = element.closest("md-dialog-content").scrollTop();
                if (initialScroll3 != newScroll3) {
                    this.disableEditor();
                }
            }.bind(this));

            setTimeout(function () {
                var initialScroll4 = element.closest(".card-body-area").scrollTop();
                element.closest(".card-body-area").on('scroll', function (e) {
                    var newScroll4 = element.closest(".card-body-area").scrollTop();
                    if (initialScroll4 != newScroll4) {
                        this.disableEditor();
                    }
                }.bind(this));
            }.bind(this));

            var initialScroll5 = element.closest(".add-inline-task").scrollTop();
            element.closest(".add-inline-task").on('scroll', function (e) {
                var newScroll5 = element.closest(".add-inline-task").scrollTop();
                if (initialScroll5 != newScroll5) {
                    this.disableEditor();
                }
            }.bind(this));

            element.closest('.layer-custom-fields-container').on('scroll', function (e) {
                this.disableEditor();
            }.bind(this));

            var initialScroll6 = element.closest('html').scrollTop();
            setTimeout(function () {
                $(window).on('scroll', function (e) {
                        var newScroll6 = element.closest('html').scrollTop();
                        if (initialScroll6 != newScroll6) {
                            this.disableEditor();
                        }
                    }.bind(this));
            }.bind(this));
        }
    }

    disableEditor() {
        this.setState({ EditorEnabled: false, showPopup: false });
        var element = $(this.refs.popoverRef);
        element.closest(".layer-details-container").css('overflow', '');
        element.closest(".layer-details-container").unbind("scroll");
        element.closest(".card-body-area").unbind("scroll");
        element.closest("md-dialog-content").unbind("scroll");
        //element.closest(".sortableDashboard").unbind("scroll");
        element.closest(".add-inline-task").unbind("scroll");
        //element.closest('.layer-custom-fields-container').unbind('scroll');

    }

    onUpdateCustomField() {
        if (this.props.UpdateCustomFields) {
            this.props.UpdateCustomFields(this.state.customField);
        }
    }

    validatePosition() {
        if(this.state.isFromInfoWindow){
            let containerDetails = $('.LayersDetailInforWindow');
            let control = $(this.refs.popoverRef);

            if ((control.offset().left - containerDetails.offset().left) < 300) {
                this.setState({ popUpPosition: "right" });
            }
        }
        else{
            let control = $(this.refs.popoverRef);

            if (control.offset().left < 300) {
                this.setState({ popUpPosition: "right" });
            }
        }
    }

    render() {
        const { EditorEnabled, editableValue, canEdit, value, body, options, customField, showPopup, popUpPosition, isItemDetails, isFromInfoWindow } = this.state;

        const clickOutSideWhiteList = [
            { type: 'class', value: 'modal' },
            { type: 'class', value: 'modal-dialog' },
            { type: 'class', value: 'modal-title' },
            { type: 'class', value: 'modal-body' },
            { type: 'class', value: 'modal-header' },
            { type: 'class', value: 'modal-content' },
            { type: 'class', value: 'modal-footer' },
            { type: 'class', value: 'form-group' },
            { type: 'class', value: 'form-control' },
            { type: 'class', value: 'note-link-btn' },
            { type: 'class', value: 'note-image-btn' },
            { type: 'class', value: 'popover-content' },
            { type: 'class', value: 'link-popover-custom-container' },
            { type: 'class', value: 'icon-editor-link' },
            { type: 'class', value: 'note-icon-chain-broken' },
            { type: 'element', value: 'label' },
            { type: 'element', value: 'span', elementValue: '×' },
            { type: 'element', value: 'a', attr: 'target', attrValue: '_blank' }
        ];

        function showFullText() {
            var tmp = document.implementation.createHTMLDocument("New").body;
            tmp.innerHTML = body;
            var strBody = tmp.textContent || tmp.innerText || "";

            var text = "Not Set";
            if (strBody !== "undefined" && strBody !== null) {
                if (strBody.trim() !== "") {
                    text = body;
                }
            }
            return { __html: text };
        };

        let containter = $('.opened-from-info-window-card');
        return (
            <div key={"cf-" + customField.CustomFieldId} ref="popoverRef" id={"cf-" + customField.CustomFieldId} className="corgee_com_custom-fields-field-B  _DEL_field-B custom-field-id" >
               
                <span className="corgee_com_custom-fields-title _DEL_title">{customField.Name}</span>
                <i className="material-icons not-filterable enable-help-tooltip" data-title="This detail type is not filterable" >do_not_disturb_on</i>
                
                <div id="editable-text-label" className="click-to-edit">
                    <div className="custom-text-directive-content-listener">
                        <div className="custom-text-directive-body-content">
                            <div dangerouslySetInnerHTML={showFullText()} onClick={this.enableEditor} className={canEdit ? 'lbl-editable-input' : 'lbl-non-editable-input'}>
                            </div>
                        </div>
                    </div>
                    <div className="custom-field-action tdc-see-more-trigger" onClick={this.showMore}>
                    </div>
                </div>

                {showPopup ?
                    <div>
                        <Overlay
                            show={showPopup}
                            target={() => findDOMNode(this.refs.popoverRef)}
                            placement={popUpPosition}
                            container={ isItemDetails || isFromInfoWindow ? containter[0] : document.body}
                            rootClose={true}
                            className="corgee_com_text-editor-container"
                            onHide={(e) => console.log('onHide', e)}
                        >
                            <Popover placement={popUpPosition} id="popover-basic" style={{ width: "300px", zIndex:"100" }} autoFocus={true} className="corgee_com_text-editor-container _DEL_text-editor-container">
                                <ClickOutSide onClickOutSide={this.disableEditor} whiteList={clickOutSideWhiteList}>
                                    <div className="corgee_com_text-editor-header _DEL_text-editor-header">
                                        <div className="corgee_com_text-editor-close-control _DEL_text-editor-close-control" onClick={this.handleClick}>
                                            <i className="icon-incorrect-icon"></i>
                                        </div>
                                    </div>
                                    <div className="corgee_com_text-editor-control _DEL_text-editor-control">
                                        <RichTextEditor onChangeBody={this.onChangeSummernote} body={body} />
                                    </div>
                                    <div className="corgee_com_text-editor-footer _DEL_text-editor-footer">
                                        <button onClick={this.save} className="corgee_com_button corgee_com_button-primary _DEL_corgee-button">Save</button>
                                    </div>
                                </ClickOutSide>
                            </Popover>
                        </Overlay>
                    </div>
                    : null}
            </div>
        )

    }
}

export default InputMultiTextField;