import React from 'react';
import CustomFieldsService from '../api/CustomFieldsService';
import PropTypes from 'prop-types';
import InputNumber from 'rc-input-number';
import 'rc-input-number/dist/rc-input-number.css';
import ClickOutSide from '../globals/ClickOutSide';

import './NumericDecimalField.css';

class NumericDecimalField extends React.Component{

    constructor(props){
        super(props);
        this.state = {
            value: (props.customField.Value === null || props.customField.Value === undefined) ? "" : props.customField.Value,
            EditorEnabled: props.EditorEnabled,
            editableValue: "",
            disableAutoSave: props.disableAutoSave,
            customField: props.customField,
            canEdit: props.canEdit && props.customField.ConversationIsEnable,
            apiConfig: props.apiConfig            
        }
        
        this.save = this.save.bind(this);
        this.enableEditor = this.enableEditor.bind(this);
        this.disableEditor = this.disableEditor.bind(this);
        this.onUpdateCustomField = this.onUpdateCustomField.bind(this);
        this.onChange = this.onChange.bind(this);
    }
    
    componentWillReceiveProps(nextProps){        
        if ((nextProps.customField.Value !== this.state.customField.Value)
            || (nextProps.customField.CustomFieldValueId !== this.state.customField.CustomFieldValueId)){
            this.setState({customField:nextProps.customField, value:nextProps.customField.Value === null ? "" : nextProps.customField.Value});                        
        }
    }

    onChange(inputValue){        
        this.setState({ value: inputValue });
    }    

    save() {        
        const {customField, value, editableValue, disableAutoSave, apiConfig} = this.state;        

        var copyCustomField = Object.assign({}, this.state.customField);
        
        // We make sure that the new value is different to the previous one
        if (value !== editableValue && value != null ) {

            // Assign the new values
            copyCustomField.Value = value;
            this.setState({editableValue:value});
            
            this.disableEditor();

            //Do not save if autoSaved is disabled
            if (disableAutoSave == "false" || disableAutoSave === undefined || !disableAutoSave) {

                let requestSave = {
                    api: this.props.apiConfig,
                    request: copyCustomField
                }; 

                if (copyCustomField.CustomFieldValueId === undefined || copyCustomField.CustomFieldValueId === null || copyCustomField.CustomFieldValueId === 0) {

                    CustomFieldsService.savePost(requestSave).then(function(data){
                        copyCustomField.CustomFieldValueId = data.CustomFieldValueId;
                                                
                        try{
                            this.setState({ customField: copyCustomField }, ()=>{
                                this.onUpdateCustomField();
                            });                           
                        }
                        catch( err )
                        {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));

                } else {

                    CustomFieldsService.save(requestSave).then(function(data){
                        try{
                            this.setState({ customField: copyCustomField }, () => {
                                this.onUpdateCustomField();
                            });                            
                        }
                        catch (err) {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));
                }
            }
            else {
                // Disable the control
                this.disableEditor();
                
                this.setState({ customField: copyCustomField }, () => {
                    this.onUpdateCustomField();
                });
            }
        } else {
            // Disable the control
            this.disableEditor();
        }
    }

    enableEditor(){
        const {canEdit, customField, value} = this.state;

        if (canEdit && customField.ConversationIsEnable) {
            this.setState({EditorEnabled:true, editableValue:value}, ()=>{
                if (this.refs.inputDecimal){
                    this.refs.inputDecimal.focus();
                }
            });            
        }
    }

    disableEditor(){
        this.setState({EditorEnabled:false});
    }

    onUpdateCustomField(){
        if(this.props.UpdateCustomFields){
            this.props.UpdateCustomFields(this.state.customField);
        }
    }

    render() {
        const {EditorEnabled, editableValue, canEdit, value, options, customField} = this.state;

        return (
            <div key={"cf-" + customField.CustomFieldId} className="corgee_com_custom-fields-field-A _DEL_field-A custom-field-id" >
                    <span className="corgee_com_custom-fields-title _DEL_title">{customField.Name}</span>

                <div className="click-to-edit">
                    {!EditorEnabled ? <div onClick={this.enableEditor} className={canEdit ? 'lbl-editable-input' : 'lbl-non-editable-input'}>
                            { value != undefined && value != "" ? value : "Not Set"}
                        </div> : null}

                    {EditorEnabled ? 
						<ClickOutSide onClickOutSide={this.save} /*whiteList={clickOutSideWhiteList}*/ >
                            <div className="corgee_com_custom-fields-numeric-decimal">
                                <div className="ctrl-editable-input" style={{position: 'absolute', zIndex: 100}}>
                                <InputNumber
                                    min={0}
                                    step={0.1}
                                    value={this.state.value}                            
                                    onChange={this.onChange}
                                    onBlur={this.save}
                                    autoFocus={true}                                                    
                                    formatter={(x)=>{                                
                                        return x.toString().replace(/(?!-)[^0-9.]/g, "");
                                    }}                            
                                />
                                </div> 
                            </div>
						</ClickOutSide>
						: null}
                    </div>
                </div>
            )
    }
}

NumericDecimalField.propTypes = {    
    customField: PropTypes.object,    
};

export default NumericDecimalField;