import React from 'react';
import CustomFieldsService from '../api/CustomFieldsService';
import $ from 'jquery';
import InputNumber from 'rc-input-number';
import 'rc-input-number/dist/rc-input-number.css';
import ClickOutSide from '../globals/ClickOutSide';

import './NumericIntegerField.css';

class NumericIntegerField extends React.Component{

    constructor(props){
        super(props);
        this.state = {
            // value: props.customField.Value === null ? "" : props.customField.Value,
            value: (props.customField.Value === null || props.customField.Value === undefined) ? "" : props.customField.Value,
            EditorEnabled: props.EditorEnabled,
            editableValue: "",
            disableAutoSave: props.disableAutoSave,
            customField: props.customField,
            canEdit: props.canEdit && props.customField.ConversationIsEnable,
            apiConfig: props.apiConfig
        }

        this.handleChange = this.handleChange.bind(this);
        this.save = this.save.bind(this);
        this.enableEditor = this.enableEditor.bind(this);
        this.disableEditor = this.disableEditor.bind(this);
        this.onUpdateCustomField = this.onUpdateCustomField.bind(this);
    }

    componentWillReceiveProps(nextProps){
        this.setState({customField:nextProps.customField, value:nextProps.customField.Value === null ? "" : nextProps.customField.Value});
    }

    handleChange(inputValue){
        // this.setState({value: event.target.value});
        this.setState({ value: inputValue });
    }

    save() {

        const {customField, value, editableValue, disableAutoSave, apiConfig} = this.state;

        var copyCustomField = Object.assign({}, this.state.customField);
        
        // We make sure that the new value is different to the previous one
        if (value !== editableValue && value != null ) {

            // Assign the new values
            copyCustomField.Value = value;
            this.setState({editableValue:value});
            
            this.disableEditor();

            //Do not save if autoSaved is disabled
            if (disableAutoSave == "false" || disableAutoSave === undefined || !disableAutoSave) {

                let requestSave = {
                    api: this.props.apiConfig,
                    request: copyCustomField
                }; 

                if (copyCustomField.CustomFieldValueId === undefined || copyCustomField.CustomFieldValueId === null || copyCustomField.CustomFieldValueId === 0) {

                    CustomFieldsService.savePost(requestSave).then(function(data){
                        copyCustomField.CustomFieldValueId = data.CustomFieldValueId;
                                                
                        try{
                            this.setState({customField: copyCustomField});

                            setTimeout(function(){
                                this.onUpdateCustomField();
                            }.bind(this));
                        }
                        catch( err )
                        {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));

                } else {

                    CustomFieldsService.save(requestSave).then(function(data){
                        try{
                            this.setState({customField: copyCustomField});

                            setTimeout(function(){
                                this.onUpdateCustomField();
                            }.bind(this));
                        }
                        catch (err) {
                            console.log(err);
                        }
                        //customFieldValueService.updateCFV(copyCustomField, 1,null, element);
                    }.bind(this));

                }
            }
            else {
                // Disable the control
                this.disableEditor();
                
                this.setState({customField: copyCustomField});
                setTimeout(function(){
                    this.onUpdateCustomField();
                }.bind(this));
            }
        } else {
            // Disable the control
            this.disableEditor();
        }
    }

    enableEditor(e){
        e.preventDefault();

        const {canEdit, customField, value} = this.state;
        if (canEdit && customField.ConversationIsEnable) {
            this.setState({EditorEnabled:true, editableValue:value});
            // setTimeout(function(){
            //     this.refs.inputInteger.focus();
            // }.bind(this),100);
            this.setState({EditorEnabled:true, editableValue:value}, ()=>{
                if (this.refs.inputInteger){
                    this.refs.inputInteger.focus();
                }
            });  
    }
    }

    disableEditor(){
        this.setState({EditorEnabled:false});
    }

    onUpdateCustomField(){
        if(this.props.UpdateCustomFields){
            this.props.UpdateCustomFields(this.state.customField);
        }
    }

    render() {
        const {EditorEnabled, editableValue, canEdit, value, options, customField} = this.state;

        return (
            <div key={"cf-"+ customField.CustomFieldId} className="corgee_com_custom-fields-field-A _DEL_field-A custom-field-id" >
                <span className="corgee_com_custom-fields-title _DEL_title">{customField.Name}</span>

                <div className="click-to-edit">
                    {!EditorEnabled ? <div onClick={this.enableEditor} className={canEdit ? 'lbl-editable-input': 'lbl-non-editable-input'}>
                        { value != undefined && value != "" ? value : "Not Set"}
                    </div> : null}

                    {EditorEnabled ? 
					<ClickOutSide onClickOutSide={this.save} /*whiteList={clickOutSideWhiteList}*/ >
                        <div className="corgee_com_custom-fields-numeric-integer">
                            <div className="ctrl-editable-input" style={{position: 'absolute', zIndex: 100}}>
                                {/* <input type="number" ref="inputInteger" pattern="\d*" value={value} min="0" step="1" onChange={this.handleChange} onBlur={this.save} /> */}
                                <InputNumber
                                    min={0}
                                    step={1}
                                    value={this.state.value}                            
                                    onChange={this.handleChange}
                                    onBlur={this.save}
                                    autoFocus={true}                                                    
                                    formatter={(x)=>{                                
                                        return x.toString().replace(/(?!-)[^0-9.]/g, "");
                                    }}                            
                                />
                            </div> 
                        </div>
					</ClickOutSide>
					: null}
                </div>
            </div>
            )

    }
}

export default NumericIntegerField;