import React, { Component } from 'react';
import PropTypes from 'prop-types';
import $ from 'jquery';

class ClickOutSide extends Component {
    constructor(props) {
        super(props);

        this.setWrapperRef = this.setWrapperRef.bind(this);
        this.handleClickOutside = this.handleClickOutside.bind(this);
    }

    handleClickOutside(event) {

        if (this.wrapperRef && !this.wrapperRef.contains(event.target)) {

            if (this.props.whiteList && this.props.whiteList.length > 0) {
                const matches = this.props.whiteList.filter(item => {
                    const value = item.type === 'class' ? `.${item.value}` : item.value;

                    if (item.elementValue) {
                        return $(event.target)[0].textContent == item.elementValue;
                    }

                    if (item.attr && item.attrValue) {
                        return $(event.target).attr(item.attr) == item.attrValue;
                    }

                    return $(event.target).is(value);
                });

                if (matches.length > 0) {
                    return;
                }
            }

            this.props.onClickOutSide();
        }
    }

    setWrapperRef(node) {
        this.wrapperRef = node;
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutside);
    }

    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutside);
    }

    render() {
        return (
            <div ref={this.setWrapperRef}>
                {this.props.children}
            </div>
        )
    }
}

ClickOutSide.propTypes = {
    whiteList: PropTypes.arrayOf(PropTypes.object),
    onClickOutSide: PropTypes.func.isRequired
}

export default ClickOutSide;