import React from 'react';
import PropTypes from 'prop-types';

import Select from 'react-select';
import 'react-select/dist/react-select.css';
// import { RenderToBody } from "./AppendToBody";
import RenderInBody from './RenderInBody';
import shortid from 'shortid';
import CorgeeStorageHelper from './CorgeeStorageHelper';
import { getVisibleLayers, getLayersName } from './ArrayUtil';
import * as apiHelper from '../api/Utils';
import { isArray } from 'util';

import './ConversationLayers.css';
import ClickOutSide from '../globals/ClickOutSide';

/*  
- Cuando hay cambios en groups sec se manda a llamar esto para sincronizar, es como con projects y layers para tareas no projects
    importante revisar: $rootScope.$on('update:GroupLayersEditable', function (e, newValue, dashboardGroupId)
-  
*/

class ConversationGroupLayers extends React.Component {
    constructor(props, context) {
        super(props, context);        

        this.state = {
            canEdit: false,
            editorEnabled: false,            
            isLoading: false,
            projectLayers: [],
            selectedIds: [],
            selectedNames: "Not Set",  
            selectedLayers: [],
            originalSelectedIds: [],
            isItemDetails: (window.location.href.indexOf("ConversationDetails") > -1) ? true : false,
            elementId: shortid.generate()
        }
        
        this.onEnableEditor = this.onEnableEditor.bind(this);
        this.onSelectGroupLayers = this.onSelectGroupLayers.bind(this);
        this.onBlurSelect = this.onBlurSelect.bind(this);
        this.fixScrollSections = this.fixScrollSections.bind(this);
        this.saveChanges = this.saveChanges.bind(this);
        this.onChangeGroupsLayers = this.onChangeGroupsLayers.bind(this);
        this.onApplyStyle = this.onApplyStyle.bind(this);
        this.onCloseSelect = this.onCloseSelect.bind(this);
    }

    componentDidMount() {
        const { conversation, isNew } = this.props;

        let canEditValidated = false;
        if (conversation.IsEnable)
            canEditValidated = CorgeeStorageHelper.canEdit() && conversation.IsEnable;
        else {
            canEditValidated = CorgeeStorageHelper.canEdit() && isNew;;
            //$scope.conversationItem.IsEnable = $scope.isNew;
        }
        let selectedValues = [], selectedNames= "Not Set";
        if (isArray(conversation.ConversationLayers) && conversation.ProjectId){
            selectedValues = conversation.ConversationLayers.map(function (item) { return item.LayerId });
            selectedNames = getLayersName(conversation.ConversationLayers);
        }

        this.setState({ canEdit: canEditValidated, 
            selectedIds: selectedValues, 
            selectedNames: selectedNames,
            originalSelectedIds: selectedValues,  
        }); 

        if (this.refs.corgeeConversationGroupLayers){
            let element = $(this.refs.corgeeConversationGroupLayers);
            this.fixScrollSections(element);            
        }
    }

    onChangeParentLayers = (conversationLayers) => {
        let selectedValues = [], selectedNames = "Not Set";
        if (isArray(conversationLayers) && conversationLayers.length>0) {
            selectedValues = conversationLayers.map(function (item) { return item.LayerId });
            selectedNames = getLayersName(conversationLayers);

            this.setState({                
                selectedIds: selectedValues,
                selectedNames: selectedNames,
                originalSelectedIds: selectedValues,
            }); 
        }
    }

    onChangeGroupsLayers(groupsLayers, stopPropagation){
        if (stopPropagation===undefined){
            stopPropagation = false
        }
        let selectedValues = [], selectedNames = "Not Set";
        let layers = [];
        let defaultLayer;

        let conversationCopy = Object.assign({}, this.props.conversation);

        if (isArray(groupsLayers)) {
            $.each(groupsLayers, function (index, item) {
                $.each(item.GroupLayers, function (ind, value) {
                    let exist = false;
                    $.each(layers, function (ind, x) {
                        if (x.LayerId == value.LayerId)
                            exist = true;
                    });
                    if (!exist)
                        layers.push(value);
                });
            });

            let finalDataLayers = getVisibleLayers(layers);
            this.setState({ projectLayers: finalDataLayers, dataGroupLayers: finalDataLayers });

            let userLayers = CorgeeStorageHelper.getAllLayers();
            let layerDefaultId = parseInt(CorgeeStorageHelper.layerId());
            
            if (layerDefaultId){
                defaultLayer = finalDataLayers.find(function (item) { return item.LayerId === layerDefaultId});
            }
            if (!defaultLayer){
                defaultLayer = finalDataLayers.find(function (item) { return item.LayerId === 1 });
            }

            if (!defaultLayer) {
                defaultLayer = finalDataLayers[0]
            }                        

            if (defaultLayer){
                selectedValues = [defaultLayer.LayerId]; // layers.map(function (item) { return item.LayerId });
                selectedNames = [defaultLayer.Name] //getLayersName(layers);  

                conversationCopy.ConversationLayers = [defaultLayer];
            }          
        }

        this.setState({            
            selectedIds: selectedValues,
            selectedNames: selectedNames,
            originalSelectedIds: selectedValues,
        });   
        
        if (stopPropagation===false && this.props.onChangedGroupLayers) {                        
            conversationCopy.ConversationGroups = (groupsLayers) ? groupsLayers : [];
            if (groupsLayers===undefined){
                conversationCopy.ConversationLayers = [];
            }
            this.props.onChangedGroupLayers(undefined, conversationCopy);
        }
    }

    onEnableEditor(e){
        $('.help-tooltip-item').remove();

        const { conversation, isNew } = this.props;

        if (isNew) { 
            /*
            if (conversation.ConversationGroups){
                let groupIds = conversation.ConversationGroups.map((item) => { return item.GroupId });

                this.setState({ isLoading: true });   

                let params = "?GroupIds=" + groupIds.join(",");
                
                params += (conversation.ConversationId && conversation.ConversationId>0) ? "&ConversationId=" + conversation.ConversationId : "";

                apiHelper.get(this.props.apiConfig, "api/layer/GetLayersByGroupIds" + params).then((dataResponse) => {
                    let finalDataLayers = getVisibleLayers(dataResponse);
                    this.setState({ projectLayers: dataResponse, dataGroupLayers: finalDataLayers, isLoading: false });
                });
            }*/
            this.setState({ editorEnabled: true });   
        } else {
            if (this.props.onChangedGroupLayers) {                
                this.props.onChangedGroupLayers(e, this.props.conversation);
            }
        }
    }

    onApplyStyle(){        
        let containerElement = $(this.refs.corgeeConversationGroupLayers);
        let targetElement = containerElement.find("#select-target-" + this.state.elementId);

        let selectElement = $("#select-render-" + this.state.elementId);
        selectElement.find('.Select').first().attr("style", "position:fixed; top: " + targetElement.offset().top + "px; left: " + targetElement.offset().left+"px");                
                        
        containerElement.closest(".layer-custom-fields-container").first().on("scroll", this.onCloseSelect);
    }

    onCloseSelect(){
        this.setState({ editorEnabled: false });

        //let containerElement = $(this.refs.corgeeConversationGroupLayers);
        //containerElement.closest(".layer-custom-fields-container").first().off("scroll wheel DOMMouseScroll mousewheel", this.onCloseSelect);
    }

    onSelectGroupLayers(itemsSelected){        
        let selectedValues = itemsSelected.map(function (item) { return item.LayerId });
        let selectedNames = getLayersName(itemsSelected); // itemsSelected.map(function (item) { return item.LayerName });

        this.setState({ selectedLayers: itemsSelected, selectedIds: selectedValues, selectedNames: selectedNames }); 
    }

    onBlurSelect(e){
        const { conversation } = this.props;
        const { selectedLayers, selectedIds, selectedNames } = this.state;

        if (this.refs.corgeeConversationGroupLayers) {
            let element = $(this.refs.corgeeConversationGroupLayers);
            element.closest(".layer-details-container").unbind("scroll");
            element.closest(".card-body-area").unbind("scroll");
            element.closest("md-dialog-content").unbind("scroll");
            //element.closest(".sortableDashboard").unbind("scroll");
            //element.closest('.layer-custom-fields-container').unbind('scroll');
            $('body').attr('style', '');
        }
        
        //if (conversation.ConversationLayers && conversation.ConversationLayers.length > 0) {}

        if (selectedLayers){
            let conversationCopy = Object.assign({}, conversation);

            conversationCopy.ConversationGroupLayers = selectedLayers;
            conversationCopy.GroupLayersValue = selectedNames;
            conversationCopy.selectedLayers = selectedIds.reverse();
            conversationCopy.ConversationLayers = selectedLayers;
            
            if (this.props.isNew === true && selectedLayers.length > 0) {                
                if (this.props.onChangedGroupLayers) {
                    this.props.onChangedGroupLayers(e, conversationCopy);
                }
            } else if(selectedLayers.length){
                this.saveChanges(e, conversationCopy);            
            }
        }

        //DE QUE TRATA?
        //GroupService.SetGroupsAndLayersForDGId($scope.conversationItem.dashboardGroupId, null, null, $scope.conversationItem.ConversationGroupLayers);

        this.setState({ editorEnabled: false });
    }

    saveChanges(e, itemCopy){
        
        let layersSelected = Object.assign([], this.state.selectedLayers);
        
        if (!this.props.disableAutoSave) {
            if (this.state.selectedIds.sort().toString() !== this.state.originalSelectedIds.sort().toString()) {

                itemCopy.PreviousLayers = this.state.originalSelectedIds.join(', ');

                for (let index = 0; index < layersSelected.length; index++) {
                    const element = layersSelected[index];

                    layersSelected[index].LayerName = layersSelected[index].Name;
                    layersSelected[index].LayerColor = layersSelected[index].Color || layersSelected[index].LayerColor;
                }
                itemCopy.ConversationLayers = layersSelected;

                apiHelper.put(this.props.apiConfig, "api/conversation/UpdateLayers", itemCopy).then((response) => {
                    //updateColors
                    let conversationLayers = [];
                    if (itemCopy) {
                        for (let i = 0; i < response.length; i++) {
                            conversationLayers.push({
                                ConversationId: itemCopy.ConversationId,
                                IsDefault: false,
                                LayerColor: response[i].LayerColor,
                                LayerId: response[i].LayerId,
                                LayerName: response[i].LayerName,
                                Color: response[i].LayerColor
                            });
                        }

                        //Update new items selected and send to parent
                        itemCopy.ConversationLayers = conversationLayers;
                        itemCopy.PreviousLayers = this.state.selectedIds.join(', ');

                        if (this.props.onChangedGroupLayers) {
                            this.props.onChangedGroupLayers(e, itemCopy);
                        }
                    }

                    this.setState({ originalSelectedIds: this.state.selectedIds });
                }).catch((error) => {
                    console.warn("Api Error", error);
                });;
            }
        }
        this.onCloseSelect();  
    }

    fixScrollSections(element){
        //console.log('fixScrollSections', element);            

        let initialScroll = element.closest(".layer-details-container").scrollTop();

        element.closest(".layer-details-container").on('scroll', function (e) {

            let newScroll = element.closest(".layer-details-container").scrollTop();
            if (initialScroll != newScroll) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        let initialScroll2 = element.closest(".sortableDashboard").scrollTop();
        element.closest(".sortableDashboard").on('scroll', function (e) {

            let newScroll2 = element.closest(".sortableDashboard").scrollTop();
            if (initialScroll2 != newScroll2) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        let initialScroll3 = element.closest("md-dialog-content").scrollTop();
        element.closest("md-dialog-content").on('scroll', function (e) {

            let newScroll3 = element.closest("md-dialog-content").scrollTop();
            if (initialScroll3 != newScroll3) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        let initialScroll4 = element.closest(".card-body-area").scrollTop();
        element.closest(".card-body-area").on('scroll', function (e) {

            let newScroll4 = element.closest(".card-body-area").scrollTop();
            if (initialScroll4 != newScroll4) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        element.closest('.layer-custom-fields-container').on('scroll', function (e) {
            this.setState({ editorEnabled: false });
        }.bind(this));

        // if (!this.state.isItemDetails){
        //     $('body').attr('style', 'overflow: hidden!important');
        // }
    }

    render() {
        const { canEdit, editorEnabled, selectedIds, selectedNames, dataGroupLayers, isLoading } = this.state;

        return (
            <div className="click-to-edit" ref="corgeeConversationGroupLayers">
                {!canEdit && <div className="lbl-non-editable-input">{selectedNames}</div>}
                {canEdit && 
                    (   editorEnabled===true ?                        
                        <div id={"select-target-" + this.state.elementId} >
                            {/* <RenderToBody>                         */}
                            <RenderInBody>
                                <div id={"select-render-" + this.state.elementId} className="comboBoxRef ConversationLayers-hidden-control-container">
								<ClickOutSide onClickOutSide={this.onBlurSelect} /*whiteList={clickOutSideWhiteList}*/ >
                                <Select ref="editableMultiSelectGroupLayer"
                                    style={{ width: 200 }}
                                    value={selectedIds}
                                    options={dataGroupLayers}                                
                                    multi={true}                                
                                    openOnFocus={true}
                                    closeOnSelect={false}
                                    clearable={false}
                                    onOpen={this.onApplyStyle}
                                    autoFocus={true}
                                    labelKey={"Name"}
                                    valueKey={"LayerId"}
                                    onChange={this.onSelectGroupLayers}
                                    onBlur={this.onBlurSelect}
                                    optionClassName="Select-option-tether"
                                />
								</ClickOutSide>
                                </div>
                            </ RenderInBody>
                            {/* </RenderToBody> */}
                        </div>                    
                    :
                    <div className="lbl-editable-input project-changed-layers" onClick={this.onEnableEditor}>{selectedNames}</div>                        
                    )
                }
            </div>
        );
    }
}

ConversationGroupLayers.propTypes = {    
    apiConfig: PropTypes.object.isRequired,    
    itemType: PropTypes.number,
    conversation: PropTypes.object.isRequired,
    disableAutoSave: PropTypes.bool,
    isNew: PropTypes.bool,
    onChangedGroupLayers: PropTypes.func
};

ConversationGroupLayers.defaultProps = {
    disableAutoSave: false,
    isNew: false
};

export default ConversationGroupLayers;