import React from 'react';
import PropTypes from 'prop-types';
import Select from 'react-select';
import 'react-select/dist/react-select.css';
import CorgeeStorageHelper from './CorgeeStorageHelper';
import { getVisibleLayers, getLayersName } from '../globals/ArrayUtil';
import * as apiHelper from '../api/Utils';
import shortid from 'shortid';
// import { RenderToBody } from "./AppendToBody";
import RenderInBody from './RenderInBody';
import './ConversationLayers.css';
import ClickOutSide from '../globals/ClickOutSide';

class ConversationLayers extends React.PureComponent {
    constructor(props, context) {
        super(props, context);
        
        let isRecycleBin = (window.location.href.indexOf("RecycleBin") > -1) ? true : false;

        let canEditFixed = false;
        if (props.conversation){
            //if(props.conversation.ConfigValueId===9){
            //canEditFixed = !CorgeeStorageHelper.isUserExternal() && props.conversation.IsEnable
            //} else {
            if (!props.conversation.IsEnable && isRecycleBin == true)
                canEditFixed = CorgeeStorageHelper.canEdit() && props.conversation.IsEnable;
            else
                canEditFixed = CorgeeStorageHelper.canEdit()
            //}
        }

        let personLayers = CorgeeStorageHelper.getAllLayers();
        if (personLayers.length == 1 && !props.isNew) {
            canEditFixed = false;
        }

        this.state = {  
            canEdit: canEditFixed,
            editorEnabled: false,
            layersSelected: [],
            selectedIds: [],
            selectedNames: "Not Set",
            configValueId: props.conversation.selectedConfigValueId ? props.conversation.selectedConfigValueId : props.conversation.ConfigValueId,
            projectId: props.conversation.ProjectID ? props.conversation.ProjectID : props.conversation.ProjectId,
            layersToHide: CorgeeStorageHelper.getLayersToHide(),
            isRecycleBin: isRecycleBin,
            dataLayers: [],
            originalSelectedIds: [],
            originalSelectedLayers: [],
            elementId: shortid.generate(),
            isProjectView: (window.location.href.indexOf("ProjectDetails") > -1) ? true : false,
            isConversationDetailsView: (window.location.href.indexOf("ConversationDetails") > -1) ? true : false,
            isFromInfoWindow: false,
            isFromTabDetails: false
        }
        
        this.onEnableEditor = this.onEnableEditor.bind(this);
        this.onSelectLayer = this.onSelectLayer.bind(this);
        this.onUpdateOriginalSelectedIds = this.onUpdateOriginalSelectedIds.bind(this);
        this.onSaveByBlur = this.onSaveByBlur.bind(this);
        this.onApplyStyle = this.onApplyStyle.bind(this);
        this.mergeLayers = this.mergeLayers.bind(this);
        this.renderOption = this.renderOption.bind(this);
        this.addLayerToCard = this.addLayerToCard.bind(this);
        this.onCloseSelect = this.onCloseSelect.bind(this);
    }    

    componentWillReceiveProps(nextProps){

        let currentLayersIds;
        let nextLayersIds;
        let selectedValues;
        let selectedNames;
        
        if((!this.props.isNew && this.state.layersSelected.length > 0) || this.props.isNew) 
        {
            if(nextProps.conversation && nextProps.conversation.isAddLayerToCard){
                currentLayersIds = this.state.originalSelectedLayers.map((item)=>{ return item.LayerId }).sort(function (a, b) { return a - b });
                nextLayersIds = nextProps.conversation.ConversationLayers.map((item) => { return item.LayerId }).sort(function (a, b) { return a - b });
                selectedValues = nextProps.conversation.ConversationLayers.map(function (item) { return item.LayerId });
                selectedNames = getLayersName(nextProps.conversation.ConversationLayers);

                this.setState({
                    layersSelected: nextProps.conversation.ConversationLayers,
                    selectedIds: selectedValues,
                    selectedNames: selectedNames,
                    originalSelectedIds: selectedValues,
                    originalSelectedLayers: nextProps.conversation.ConversationLayers
                });

            }
            else if (this.state.layersSelected && nextProps.conversation && nextProps.conversation.ConversationLayers){
                
                currentLayersIds = this.state.originalSelectedLayers.map((item)=>{ return item.LayerId }).sort(function (a, b) { return a - b });
                nextLayersIds = nextProps.conversation.ConversationLayers.map((item) => { return item.LayerId }).sort(function (a, b) { return a - b });
                
                if (JSON.stringify(currentLayersIds) !== JSON.stringify(nextLayersIds)){
                    selectedValues = nextProps.conversation.ConversationLayers.map(function (item) { return item.LayerId });
                    selectedNames = getLayersName(nextProps.conversation.ConversationLayers);

                    this.setState({
                        layersSelected: nextProps.conversation.ConversationLayers,
                        selectedIds: selectedValues,
                        selectedNames: selectedNames,
                        originalSelectedIds: selectedValues,
                        originalSelectedLayers: nextProps.conversation.ConversationLayers
                    });
                }            
            }
        }
        else{
            currentLayersIds = this.state.originalSelectedLayers.map((item)=>{ return item.LayerId }).sort(function (a, b) { return a - b });
            
            selectedValues = this.state.originalSelectedLayers.map(function (item) { return item.LayerId });
            selectedNames = getLayersName(this.state.originalSelectedLayers);

            this.setState({
                layersSelected: this.state.originalSelectedLayers,
                selectedIds: selectedValues,
                selectedNames: selectedNames,
                originalSelectedIds: selectedValues,
                originalSelectedLayers:this.state.originalSelectedLayers
            });
        }
            
        if (this.state.configValueId && nextProps.conversation && nextProps.conversation.selectedConfigValueId
            && this.state.configValueId !== nextProps.conversation.selectedConfigValueId) 
            {
            this.setState({ configValueId: nextProps.conversation.selectedConfigValueId });
        }
        if (this.state.projectId && nextProps.conversation && nextProps.conversation.ProjectId
            && this.state.projectId !== nextProps.conversation.ProjectId) {
            this.setState({ projectId: nextProps.conversation.ProjectId });
        }
    }

    componentDidMount() {
        const { conversation } = this.props;

        if (conversation && conversation.ConversationLayers){
            let selectedValues = conversation.ConversationLayers.map(function (item) { return item.LayerId });
            let selectedNames = getLayersName(conversation.ConversationLayers);            

            this.setState({                 
                    layersSelected: conversation.ConversationLayers, 
                    selectedIds: selectedValues,
                    selectedNames: selectedNames,
                    originalSelectedIds: selectedValues, 
                    originalSelectedLayers: conversation.ConversationLayers
            });
        }        
        if (this.refs.corgeeConversationLayers){
            let containerElement = $(this.refs.corgeeConversationLayers);
            if (containerElement.closest(".details-tab-directive").length > 0) {
                this.setState({isFromInfoWindow: true});
            }
            if(containerElement.closest('#canvasDetails').length>0){
                this.setState({ isFromTabDetails: true });
            }
        }
    }

    onEnableEditor(e){        
        const { configValueId, projectId } = this.state;
        const { conversation } = this.props;        

        let params = "?";
        if (configValueId){
            params += "configValueId=" + configValueId;
        }
        if (conversation && conversation.ProjectId) {
            params += "&projectId=" + conversation.ProjectId;
        }
        if (conversation && conversation.ProjectID) {
            params += "&projectId=" + conversation.ProjectID;
        }
        if (conversation && conversation.ConversationId) {
            params += "&conversationId=" + conversation.ConversationId;
        }
        if(!conversation.ProjectId && !conversation.ProjectID) conversation.ProjectConversationId = undefined;

        apiHelper.get(this.props.apiConfig, "api/layer/GetLayersByConfigValueId" + params).then((dataResponse)=>{
            
            let dataLayers = dataResponse;

            if (conversation.selectedConfigValueId !== 9 && conversation.ConfigValueId !== 9) {
                if (conversation.ProjectConversationId && conversation.ProjectConversationId != 0) {
                    let userLayers = CorgeeStorageHelper.getAllLayers();
                    dataLayers = this.mergeLayers(dataResponse, userLayers, conversation.ProjectConversationId);
                }
            }

            let finalDataLayers = getVisibleLayers(dataLayers);            
            this.setState({ dataLayers: finalDataLayers});

            $('body').css('overflow', 'hidden');
            
            let element = $(this.refs.corgeeConversationLayers);
            let window = $(".conversation-editor");

            if(window.length > 0){
                element.closest(".layer-details-container").css('overflow', 'hidden');
            }
            
            var infoWindowScrollContainer = element.closest('.details-tab-directive .card-body-area');
            if (infoWindowScrollContainer.length > 0) {
                infoWindowScrollContainer.css('overflow', '');
            }        
    
            var initialScroll2 = element.closest(".sortableDashboard").scrollTop();
            element.closest(".sortableDashboard").on('scroll', function (e) {
                var newScroll2 = element.closest(".sortableDashboard").scrollTop();
                if (initialScroll2 != newScroll2) {
                    this.onSaveByBlur();
                }
            }.bind(this));
    
            var initialScroll3 = element.closest("md-dialog-content").scrollTop();
            element.closest("md-dialog-content").on('scroll', function (e) {
                var newScroll3 = element.closest("md-dialog-content").scrollTop();
                if (initialScroll3 != newScroll3) {
                    this.onSaveByBlur();
                }
            }.bind(this));
    
            setTimeout(function(){
                var initialScroll4 = element.closest(".card-body-area").scrollTop();
                element.closest(".card-body-area").on('scroll', function (e) {
                    var newScroll4 = element.closest(".card-body-area").scrollTop();
                    if (initialScroll4 != newScroll4) {
                        this.onSaveByBlur();
                    }
                }.bind(this));
            }.bind(this));
            
            var initialScroll5 = element.closest(".add-inline-task").scrollTop();
            element.closest(".add-inline-task").on('scroll', function (e) {
                var newScroll5 = element.closest(".add-inline-task").scrollTop();
                if (initialScroll5 != newScroll5) {
                    this.onSaveByBlur();
                }
            }.bind(this));
    
            element.closest('.layer-custom-fields-container').on('scroll', function (e) {
                this.onSaveByBlur();
            }.bind(this));        

        }).catch((error) => {
            console.warn("Api Error", error);
        });; 

        this.setState({ editorEnabled: true });
    }

    onSelectLayer(itemsSelected){

        itemsSelected = itemsSelected.sort(function (a, b) { return a.LayerId - b.LayerId });

        let selectedValues = itemsSelected.map(function (item) { return item.LayerId });
        let selectedNames = getLayersName(itemsSelected);

        this.setState({ layersSelected: itemsSelected, selectedIds: selectedValues, selectedNames: selectedNames}); 
    }

    onUpdateOriginalSelectedIds(itemsSelected){
        itemsSelected = itemsSelected.sort(function (a, b) { return a.LayerId - b.LayerId });
        let selectedValues = itemsSelected.map(function (item) { return item.LayerId });
        this.setState({originalSelectedIds: selectedValues}); 
    }

    onSaveByBlur(e){
        $('.help-tooltip-item').remove();        

        let element = $(this.refs.corgeeConversationLayers);
        element.closest(".layer-details-container").css('overflow', '');
        $('body').css('overflow', '');

        var infoWindowScrollContainer = element.closest('.details-tab-directive .card-body-area')
        if (infoWindowScrollContainer.length > 0) {
            infoWindowScrollContainer.css('overflow', '');
        }

        element.closest(".layer-details-container").unbind("scroll");
        element.closest(".card-body-area").unbind("scroll");
        element.closest("md-dialog-content").unbind("scroll");
        //element.closest(".sortableDashboard").unbind("scroll");
        element.closest(".add-inline-task").unbind("scroll");
        //element.closest('.layer-custom-fields-container').unbind('scroll');


        if (this.state.selectedIds.sort().toString() !== this.state.originalSelectedIds.sort().toString()) {
            //SAVE NEW LAYERS SELECTED

            if(this.props.isNew){
                //Execute $scope.$emit('inline:layer:changed', $scope.conversationItem);
                let itemCopy = Object.assign({}, this.props.conversation);
                let layersSelected = Object.assign([], this.state.layersSelected);
                itemCopy.ConversationLayers = layersSelected;                

                if (this.props.onChangedLayers) {
                    this.props.onChangedLayers(itemCopy, this.state.isFromInfoWindow);
                }
                this.setState({ originalSelectedIds: this.state.selectedIds });
                this.setState({ originalSelectedLayers: layersSelected });
            }

            if(!this.props.disableAutoSave){         
                if((!this.props.isNew && this.state.originalSelectedIds.length > 0 && this.state.layersSelected.length > 0) || this.props.isNew) {

                
                    let itemCopy = Object.assign({}, this.props.conversation);
                    let layersSelected = Object.assign([], this.state.layersSelected);
                    
                    itemCopy.PreviousLayers = this.state.originalSelectedIds.join(', ');                
                                    
                    for (let index = 0; index < layersSelected.length; index++) { 
                        const element = layersSelected[index];

                        layersSelected[index].LayerName = layersSelected[index].Name;
                        layersSelected[index].LayerColor = layersSelected[index].Color || layersSelected[index].LayerColor;                                                
                    }
                    itemCopy.ConversationLayers = layersSelected;

                    apiHelper.put(this.props.apiConfig, "api/conversation/UpdateLayers", itemCopy).then((response) => {
                        let conversationLayers = [];
                        if (itemCopy) {
                            for (let i = 0; i < response.length; i++) { 
                                conversationLayers.push({
                                    ConversationId: itemCopy.ConversationId,
                                    IsDefault: false,
                                    LayerColor: response[i].LayerColor,
                                    LayerId: response[i].LayerId,
                                    LayerName: response[i].LayerName,
                                    Color: response[i].LayerColor
                                });
                            }

                            //Update new items selected and send to parent
                            itemCopy.ConversationLayers = conversationLayers;
                            itemCopy.PreviousLayers = this.state.selectedIds.join(', ');

                            if (this.props.onChangedLayers){
                                this.props.onChangedLayers(itemCopy, this.state.isFromInfoWindow);
                            }
                        }

                        this.setState({ originalSelectedIds: conversationLayers });
                        this.setState({ originalSelectedLayers: layersSelected });
                    }).catch((error) => {
                        console.warn("Api Error", error);
                    });
                }
                else{
                    let currentLayersIds = this.state.originalSelectedLayers.map((item)=>{ return item.LayerId }).sort(function (a, b) { return a - b });
                    
                    let selectedValues = this.state.originalSelectedLayers.map(function (item) { return item.LayerId });
                    let selectedNames = getLayersName(this.state.originalSelectedLayers);
        
                    this.setState({
                        layersSelected: this.state.originalSelectedLayers,
                        selectedIds: selectedValues,
                        selectedNames: selectedNames,
                        originalSelectedIds: selectedValues,
                        originalSelectedLayers:this.state.originalSelectedLayers
                    });
                }
            }
        }
        this.onCloseSelect();
    }    

    mergeLayers(currentLayers, userLayers, projectconversationid){
        currentLayers = currentLayers.map(function (obj, key) { return obj; });
        let result = currentLayers.concat(userLayers);

        result = result.filter(
            function (obj, index, self) {
                let result = self.findIndex(function (o) { return o.LayerId === obj.LayerId });
                return result === index
            });

        for (let i = 0; i < currentLayers.length; i++) {
            for (let j = 0; j < result.length; j++) {
                if (currentLayers[i].LayerId == result[j].LayerId) {
                    result[j].IsAvailable = true;
                    break;
                };
            };
        };

        for (let i = 0; i < result.length; i++) {
            result[i].ProjectConversationId = projectconversationid;
            if (!result[i].IsAvailable) {
                result[i].disabled = true;
            }
            /*
            if (this.props.isNew === true) {
                result[i].IsAvailable = true;                
            } else {
                if (!result[i].IsAvailable){
                    result[i].disabled = true;
                }
            } */           
        };
        return result;
    }

    addLayerToCard(e, layer){
        if (this.props.addLayerToCard) {
            this.props.addLayerToCard(e, layer);
        }
        this.setState({ editorEnabled: false });        
    }

    onApplyStyle() {
        //if (!this.state.isFromInfoWindow && !this.state.isFromTabDetails){
            let isProjectView = (window.location.href.indexOf("ProjectDetails") > -1) ? true : false;
            this.setState({ isProjectView: isProjectView} );

            let containerElement = $(this.refs.corgeeConversationLayers);            

            let targetElement = containerElement.find("#select-target-" + this.state.elementId);            
            let targetTopValue = targetElement.offset().top;                        

            if(window.scrollY>0){
                targetTopValue -= window.scrollY;
            }

            let selectElement = $("#select-render-" + this.state.elementId);            
            selectElement.find('.Select').first().attr("style", "position:fixed; top: " + targetTopValue + "px; left: " + targetElement.offset().left + "px");

            containerElement.closest(".layer-custom-fields-container").first().on("scroll", this.onCloseSelect);
        //}
    }

    onCloseSelect() {
        this.setState({ editorEnabled: false });

        //let containerElement = $(this.refs.corgeeConversationLayers);
        //containerElement.closest(".layer-custom-fields-container").first().off("scroll wheel DOMMouseScroll mousewheel", this.onCloseSelect);
    }

    renderOption(option) {        
        if (option.IsAvailable === true || !option.ProjectConversationId ){
            return (<div style={{cursor: "pointer"}}>
                <span className="name">{option.Name}</span>
            </div>);
        } else {
            return (<div style={{cursor: "default"}}>
                <span className="name">{option.Name}</span>
                    <div style={{fontSize: 10}} className="add-new-layer">
                        <span style={{fontStyle: "italic"}}>
                        (<span style={{marginRight: 4}}>Not available on this card</span> 
                        <b className="add-layer-text" onClick={(e) => this.addLayerToCard(e, option) } >Add?</b>)
                        </span>
                    </div>
                </div>);
        }
    }

    render() {
        const { canEdit, editorEnabled, selectedIds, selectedNames, dataLayers, isFromInfoWindow, isFromTabDetails, isConversationDetailsView } = this.state;

        let isProjectView = (window.location.href.indexOf("ProjectDetails") > -1) ? true : false;
        let styleRender = { position: 'fixed', zIndex: 100 };
        if (isProjectView) {
            styleRender.top = 15;
        }

        let containerSelectorId = "select-target-" + this.state.elementId;
        let containerSelectorTarget = "#subtree-container";
        if (isFromInfoWindow || isFromTabDetails){
            containerSelectorTarget = "#" + containerSelectorId;
            styleRender.fontSize = 15;    
            styleRender.position = "absolute";
        }

        return (
            <div className="click-to-edit" ref="corgeeConversationLayers">
                {!canEdit && <div className="lbl-non-editable-input">{selectedNames}</div>}
                {canEdit && 
                    (   editorEnabled===true && !isConversationDetailsView ? 
                        <div id={"select-target-" + this.state.elementId}>                      
                        {/* <RenderToBody  > */}
                        <RenderInBody>
                                <div id={"select-render-" + this.state.elementId} className="comboBoxRef ConversationLayers-hidden-control-container" style={styleRender}>
								<ClickOutSide onClickOutSide={this.onSaveByBlur} /*whiteList={clickOutSideWhiteList}*/ >
                                    <Select ref="editableMultiSelectLayer"
                                        style={{ width: 200 }}
                                        value={selectedIds}
                                        options={dataLayers}                                
                                        multi={true}                                
                                        openOnFocus={true}
                                        closeOnSelect={false}
                                        clearable={false}
                                        optionRenderer={this.renderOption}
                                        onOpen={this.onApplyStyle}
                                        autoFocus={true}
                                        labelKey={"Name"}
                                        valueKey={"LayerId"}
                                        onChange={this.onSelectLayer}
                                        placeholder = "Select a Layer..."                                
                                        onBlur={this.onSaveByBlur}
                                        optionClassName="Select-option-tether"
                                    />
								</ClickOutSide>
                                </div>
                        </ RenderInBody>
                        {/* </RenderToBody>   */}
                        </div>                  
                    :
                        editorEnabled===true && isConversationDetailsView ? 
                        <div id={"select-target-" + this.state.elementId}>                      
                            <div id={"select-render-" + this.state.elementId} className="comboBoxRef ConversationLayers-hidden-control-container" style={styleRender}>
                            <ClickOutSide onClickOutSide={this.onSaveByBlur} /*whiteList={clickOutSideWhiteList}*/ >
                                <Select ref="editableMultiSelectLayer"
                                    style={{ width: 200 }}
                                    value={selectedIds}
                                    options={dataLayers}                                
                                    multi={true}                                
                                    openOnFocus={true}
                                    closeOnSelect={false}
                                    clearable={false}
                                    optionRenderer={this.renderOption}
                                    onOpen={this.onApplyStyle}
                                    autoFocus={true}
                                    labelKey={"Name"}
                                    valueKey={"LayerId"}
                                    onChange={this.onSelectLayer}
                                    placeholder = "Select a Layer..."                                
                                    onBlur={this.onSaveByBlur}
                                    optionClassName="Select-option-tether"
                                />
                            </ClickOutSide>
                            </div>
                        </div>                  
                        :<div className="lbl-editable-input" onClick={this.onEnableEditor}>{selectedNames}</div>                        
                    )
                }
            </div>
        );
    }
}

ConversationLayers.propTypes = {    
    apiConfig: PropTypes.object.isRequired,    
    itemType: PropTypes.number,
    conversation: PropTypes.object.isRequired,
    disableAutoSave: PropTypes.bool,
    isNew: PropTypes.bool,
    onChangedLayers: PropTypes.func
};

ConversationLayers.defaultProps = {
    disableAutoSave: false,
    isNew: false
};

export default ConversationLayers;