import React from 'react';
import PropTypes from 'prop-types';
import Select from 'react-select';
import 'react-select/dist/react-select.css';
// import { RenderToBody } from "./AppendToBody";
import RenderInBody from './RenderInBody';
import shortid from 'shortid';
import CorgeeStorageHelper from './CorgeeStorageHelper';
import * as apiHelper from '../api/Utils';
import './ConversationProjects.css';
import ClickOutSide from '../globals/ClickOutSide';

class ConversationProjects extends React.PureComponent {
    constructor(props, context) {
        super(props, context);        

        this.state = {
            canEdit: false,
            editorEnabled: false,
            selectedProject: {},            
            selectedProjectName: "Not Set",            
            dataProjects: [],
            configValueId: props.conversation.selectedConfigValueId ? props.conversation.selectedConfigValueId : props.conversation.ConfigValueId,
            isLoading: false,
            previousProjectId: 0,
            elementId: shortid.generate()
        }
        
        this.onEnableEditor = this.onEnableEditor.bind(this);
        this.onSelectProject = this.onSelectProject.bind(this);
        this.onSaveByBlur = this.onSaveByBlur.bind(this); 
        this.getConvIdByProjId = this.getConvIdByProjId.bind(this);   
        this.getProjectNameById = this.getProjectNameById.bind(this); 
        this.onApplyStyle = this.onApplyStyle.bind(this);
        this.onCloseSelect = this.onCloseSelect.bind(this);
    }

    componentWillReceiveProps(nextProps) {
        if (this.state.selectedProject && nextProps.conversation) {

            let projectId = nextProps.conversation.MilestoneId ? nextProps.conversation.MilestoneId : (nextProps.conversation.ProjectId ? nextProps.conversation.ProjectId : nextProps.conversation.ProjectID);            
            this.setState({ configValueId: nextProps.conversation.selectedConfigValueId });

            if (this.state.selectedProject.ProjectId !== projectId){                
                let selectedProject = {
                    ProjectId: projectId
                };

                if (nextProps.conversation.ProjectName && nextProps.conversation.ProjectName !== "" && nextProps.conversation.ProjectName.length > 0) {
                    selectedProject.Name = nextProps.conversation.ProjectName;
                } else {
                    selectedProject.Name = "Not Set";
                }

                this.setState({
                    selectedProject: selectedProject,
                    selectedProjectName: selectedProject.Name, 
                    previousProjectId: projectId
                });
            }            
        }
    }

    componentDidMount() {
        const { conversation, isNew } = this.props;  
        
        let canEditValidated = false, isEnable;
        let isRecycleBin = (window.location.href.indexOf("RecycleBin") > -1) ? true : false;
        let selectedProject = {};

        if (conversation != undefined){
            isEnable = !conversation.IsEnable && (isRecycleBin == true) ? false : true;

            if (!isNew) {
                let projectId = conversation.MilestoneId ? conversation.MilestoneId : (conversation.ProjectId ? conversation.ProjectId : conversation.ProjectID);                        
                selectedProject.ProjectId = projectId;

                if (conversation.ProjectName && conversation.ProjectName !== "" && conversation.ProjectName.length>0){
                    selectedProject.Name = conversation.ProjectName;

                    this.setState({ selectedProjectName: selectedProject.Name });
                }
                this.setState({ previousProjectId: projectId });
            }
        } else {
            isEnable = (isRecycleBin == true) ? false : true;
        }

        canEditValidated = CorgeeStorageHelper.canEdit() && isEnable;

        if (!isNew) {
            let canEditProject = false;

            if (CorgeeStorageHelper.canEdit() && conversation !== undefined && conversation.CreatedByID === CorgeeStorageHelper.personId()) {
                canEditProject = true;
            }
            if (CorgeeStorageHelper.isAdmin()) {
                canEditProject = true;
            }

            canEditValidated = canEditProject && isEnable;
        }

        this.setState({ canEdit: canEditValidated, selectedProject: selectedProject });        
    }

    onEnableEditor(e){ 
        if(this.props.isNew){
            this.setState({ editorEnabled: true, isLoading: true });

            apiHelper.get(this.props.apiConfig, "api/project/getProjectsByConfigValueId?configValueId=" + this.state.configValueId).then((dataResponse) => {
                this.setState({ dataProjects: dataResponse, isLoading: false });                
            }).catch((error) => {
                console.warn("Api Error", error);
            });;
        } else {
            if (this.props.onChangedProject){
                const { conversation } = this.props;

                this.props.onChangedProject(e, conversation);
            }
        }
    }

    onSelectProject(selectedItem){    
        if (selectedItem){   
            this.setState({ selectedProject: selectedItem, selectedProjectName: selectedItem.Name }, this.onSaveByBlur);
        } else {
            this.setState({ selectedProject: {}, selectedProjectName: "Not Set" }, this.onSaveByBlur); 
        }
        this.onCloseSelect();
    }

    onSaveByBlur(e){
        const { selectedProject, previousProjectId } = this.state;
        
        let convItemCopy = Object.assign({}, this.props.conversation);

        if (selectedProject.ProjectId) {
            convItemCopy.ProjectId = selectedProject.ProjectId;
            convItemCopy.ProjectName = this.getProjectNameById(selectedProject.ProjectId);
            convItemCopy.ProjectName = convItemCopy.ProjectName ? convItemCopy.ProjectName  : "Not Set";
            convItemCopy.AlternateProjectName = convItemCopy.ProjectName;  
            convItemCopy.ProjectConversationId = this.getConvIdByProjId(selectedProject.ProjectId);   
        } else {
            convItemCopy.ProjectId = null;
            convItemCopy.ProjectName = "Not Set";
            convItemCopy.ConversationLayers = [];
            convItemCopy.AlternateProjectName = convItemCopy.ProjectName;
            convItemCopy.ProjectConversationId = this.getConvIdByProjId(selectedProject.ProjectId);
            
            let MyPersonLayers = CorgeeStorageHelper.getAllLayers();                        
            convItemCopy.ConversationLayers = MyPersonLayers.filter(function (e) { return e.LayerId == 1 });
        }        

        if (!this.props.disableAutoSave) { 
            let request = {
                ConversationId: convItemCopy.ConversationId,
                ProjectId: convItemCopy.ProjectId,
                PreviousProject: previousProjectId
            }

            apiHelper.put(this.props.apiConfig, "api/conversation/UpdateProject", request, true).then((response) => {
                if (this.props.onChangedProject) {
                    this.props.onChangedProject(e, convItemCopy);
                }
            }).catch((error) => {
                console.warn("Api Error", error);
            });;
        } else {
            if (this.props.onChangedProject) {
                this.props.onChangedProject(e, convItemCopy);
            }            
        }
        
        this.onCloseSelect();
    }

    getProjectNameById(id) {
        const { dataProjects } = this.state;

        for (var i = 0; i < dataProjects.length; i++) {
            if (dataProjects[i].ProjectId == id || dataProjects[i].MilestoneId == id) {
                return dataProjects[i].Name;
            }
        }
    }

    getConvIdByProjId(id) {
        const { dataProjects } = this.state;

        for (var i = 0; i < dataProjects.length; i++) {
            if (dataProjects[i].ProjectId == id || dataProjects[i].MilestoneId == id) {
                return dataProjects[i].ConversationId;
            }
        }
    }

    onApplyStyle(){        
        let containerElement = $(this.refs.corgeeConversationProjects);
        let targetElement = containerElement.find("#select-target-" + this.state.elementId);

        let selectElement = $("#select-render-" + this.state.elementId);
        selectElement.find('.Select').first().attr("style", "position:fixed; top: " + targetElement.offset().top + "px; left: " + targetElement.offset().left+"px");                
                        
        containerElement.closest(".layer-custom-fields-container").first().on("scroll", this.onCloseSelect);
    }

    onCloseSelect(){
        this.setState({ editorEnabled: false });

        //let containerElement = $(this.refs.corgeeConversationProjects);
        //containerElement.closest(".layer-custom-fields-container").first().off("scroll wheel DOMMouseScroll mousewheel", this.onCloseSelect);
    }

    render() {
        const { canEdit, editorEnabled, selectedProject, selectedProjectName, dataProjects, isLoading } = this.state;

        return (
            <div className="click-to-edit" ref="corgeeConversationProjects">
                {!canEdit && <div className="lbl-non-editable-input">{selectedProjectName}</div>}
                {canEdit && 
                    (   editorEnabled===true ?
                    <div id={"select-target-" + this.state.elementId} >
                        {/* <RenderToBody>                         */}
                        <RenderInBody>
                            <div id={"select-render-" + this.state.elementId} className="select-dropdown-container comboBoxRef">
							<ClickOutSide onClickOutSide={this.onSaveByBlur} /*whiteList={clickOutSideWhiteList}*/ >
                                <Select ref="editableMultiSelectProject"
                                    style={{ width: 200 }}
                                    value={selectedProject.ProjectId}
                                    options={dataProjects}
                                    openOnFocus={true}                            
                                    closeOnSelect={true}
                                    clearable={true}
                                    clearValueText="Remove Project"
                                    onOpen={this.onApplyStyle}
                                    autoFocus={true}
                                    labelKey={"Name"}
                                    valueKey={"ProjectId"}
                                    onChange={this.onSelectProject}                            
                                    placeholder="Select a Project..."
                                    onBlur={this.onSaveByBlur}                                
                                    optionClassName="Select-option-tether"
                                />
							</ClickOutSide>
                            </div>
                        </ RenderInBody>
                        {/* </RenderToBody>   */}
                    </div>                  
                    :
                    <div className="lbl-editable-input project-changed" onClick={this.onEnableEditor}>{selectedProjectName}</div>                        
                    )
                }
            </div>
        );
    }
}

ConversationProjects.propTypes = {    
    apiConfig: PropTypes.object.isRequired,    
    itemType: PropTypes.number,
    conversation: PropTypes.object.isRequired,
    disableAutoSave: PropTypes.bool,
    isNew: PropTypes.bool,
    onChangedProject: PropTypes.func
};

ConversationProjects.defaultProps = {
    disableAutoSave: false,
    isNew: false
};

export default ConversationProjects;