import React from 'react';
import PropTypes from 'prop-types';
import Select from 'react-select';
import 'react-select/dist/react-select.css';
// import { RenderToBody } from "./AppendToBody";
import RenderInBody from './RenderInBody';
import shortid from 'shortid';
import CorgeeStorageHelper from '../globals/CorgeeStorageHelper';
import { getGroupsNames } from '../globals/ArrayUtil';
import * as apiHelper from '../api/Utils';
import ClickOutSide from '../globals/ClickOutSide';

class ConversationSecurityGroups extends React.PureComponent {
    constructor(props, context) {
        super(props, context);        

        this.state = {
            canEdit: false,
            editorEnabled: false,            
            isLoading: false,            
            selectedIds: [],
            selectedNames: "Not Set",  
            selectedSecurityGroups: [],
            originalSelectedIds: [],
            isItemDetails: (window.location.href.indexOf("ConversationDetails") > -1) ? true : false,
            elementId: shortid.generate()
        }
        
        this.onEnableEditor = this.onEnableEditor.bind(this);
        this.onSelectSecurityGroups = this.onSelectSecurityGroups.bind(this);
        this.onBlurSelect = this.onBlurSelect.bind(this);
        this.fixScrollSections = this.fixScrollSections.bind(this);
        this.saveChanges = this.saveChanges.bind(this);
        this.setDefaultGroup = this.setDefaultGroup.bind(this);
        this.onApplyStyle = this.onApplyStyle.bind(this);
        this.onCloseSelect = this.onCloseSelect.bind(this);
    }

    componentDidMount() {
        const { conversation, isNew } = this.props;

        let canEditValidated = false;
        if (conversation.IsEnable)
            canEditValidated = CorgeeStorageHelper.canEdit() && conversation.IsEnable;
        else {
            canEditValidated = CorgeeStorageHelper.canEdit() && isNew;;            
        }
        this.setState({ canEdit: canEditValidated });

        if (conversation && conversation.ConversationGroups) {
            let selectedValues = conversation.ConversationGroups.map(function (item) { return item.GroupId });

            this.setState({                
                selectedIds: selectedValues,
                selectedNames: getGroupsNames(conversation.ConversationGroups),
                originalSelectedIds: selectedValues,
                selectedSecurityGroups: conversation.ConversationGroups
            });
        }                

        if (this.refs.corgeeConversationSecurityGroups) {
            let element = $(this.refs.corgeeConversationSecurityGroups);
            this.fixScrollSections(element);
        }        
    }

    setDefaultGroup(defaultGroup){
        if (defaultGroup){
            this.setState({
                selectedIds: [defaultGroup.GroupId],
                selectedNames: [defaultGroup.Name],
                originalSelectedIds: [defaultGroup.GroupId],
                selectedSecurityGroups: [defaultGroup]
            });

            if (this.props.onChangedSecurityGroups) {
                let conversationCopy = Object.assign({}, this.props.conversation);
                conversationCopy.ConversationGroups = [defaultGroup];

                if (this.props.onChangedSecurityGroups) {
                    this.props.onChangedSecurityGroups(undefined, conversationCopy);
                }                                                
            }
        }
    }

    componentWillReceiveProps(nextProps){
        if (this.state.selectedSecurityGroups && nextProps.conversation 
            && nextProps.conversation.ConversationGroups
            && !this.props.isNew) 
        {
            if (this.state.selectedSecurityGroups.length === 0 && nextProps.conversation.ConversationGroups.length>0){
                let selectedValues = nextProps.conversation.ConversationGroups.map(function (item) { return item.GroupId });

                this.setState({                    
                    selectedIds: selectedValues,
                    selectedNames: getGroupsNames(nextProps.conversation.ConversationGroups),
                    originalSelectedIds: selectedValues,
                    selectedSecurityGroups: nextProps.conversation.ConversationGroups
                });
            }
        }    
        
        if (this.props.conversation && nextProps.conversation
            && nextProps.conversation.ConversationGroups
            && this.props.conversation.ConversationGroups
            && nextProps.conversation.ConversationGroups.length !== this.props.conversation.ConversationGroups.length
            && !this.props.isNew) 
        {
            let selectedValues = nextProps.conversation.ConversationGroups.map(function (item) { return item.GroupId });

            this.setState({
                selectedIds: selectedValues,
                selectedNames: getGroupsNames(nextProps.conversation.ConversationGroups),
                originalSelectedIds: selectedValues,
                selectedSecurityGroups: nextProps.conversation.ConversationGroups
            });            
        }
    }

    onEnableEditor(e){
        $('.help-tooltip-item').remove();
        const { conversation, isNew } = this.props;
        
        if (this.props.isNew) {                        
            this.setState({ isLoading: true });

            apiHelper.get(this.props.apiConfig, "api/group/GetGroups").then((dataResponse) => {
                this.setState({ dataSecurityGroups: dataResponse, isLoading: false });
            }).catch((error) => {
                console.warn("Api Error", error);
            });;
            
            this.setState({ editorEnabled: true });            
        } else {
            if (this.props.onChangedSecurityGroups) {
                const { conversation } = this.props;
                this.props.onChangedSecurityGroups(e, conversation);
            }
        }
    }

    onSelectSecurityGroups(itemsSelected){        
        let selectedValues = itemsSelected.map(function (item) { return item.GroupId });
        let selectedNames = getGroupsNames(itemsSelected);

        this.setState({ selectedSecurityGroups: itemsSelected, selectedIds: selectedValues, selectedNames: selectedNames }); 
    }

    onBlurSelect(e){ 
        const { conversation } = this.props;
        const { selectedSecurityGroups, selectedIds, selectedNames } = this.state;

        if (this.refs.corgeeConversationSecurityGroups) {
            let element = $(this.refs.corgeeConversationSecurityGroups);
            element.closest(".layer-details-container").unbind("scroll");
            element.closest(".card-body-area").unbind("scroll");
            element.closest("md-dialog-content").unbind("scroll");
            //element.closest(".sortableDashboard").unbind("scroll");
            //element.closest('.layer-custom-fields-container').unbind('scroll');
            $('body').attr('style', '');
        }

        let conversationCopy = Object.assign({}, conversation);

        if (selectedSecurityGroups && selectedSecurityGroups.length) {            
            conversationCopy.ConversationGroups = selectedSecurityGroups;
            conversationCopy.GroupsValue = selectedNames;
            conversationCopy.selectedGroups = selectedIds;                
        } else {
            conversationCopy.ConversationGroups = [];
            conversationCopy.GroupsValue = "Not Set";
            conversationCopy.selectedGroups = [];  
        }
        this.saveChanges(e, conversationCopy);
        this.setState({ editorEnabled: false });
    }

    saveChanges(e, itemCopy){
        if (this.props.onChangedSecurityGroups) {
            this.props.onChangedSecurityGroups(e, itemCopy);
        }
        this.onCloseSelect();
        /*
        disableEditor();
        setLayersGroupSelected();
        customFieldValueService.updateCFV($scope.conversationItem, 0, 3);
        */             
    }

    fixScrollSections(element) {
        //console.log('fixScrollSections', element);

        let initialScroll = element.closest(".layer-details-container").scrollTop();

        element.closest(".layer-details-container").on('scroll', function (e) {

            let newScroll = element.closest(".layer-details-container").scrollTop();
            if (initialScroll != newScroll) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        let initialScroll2 = element.closest(".sortableDashboard").scrollTop();
        element.closest(".sortableDashboard").on('scroll', function (e) {

            let newScroll2 = element.closest(".sortableDashboard").scrollTop();
            if (initialScroll2 != newScroll2) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        let initialScroll3 = element.closest("md-dialog-content").scrollTop();
        element.closest("md-dialog-content").on('scroll', function (e) {

            let newScroll3 = element.closest("md-dialog-content").scrollTop();
            if (initialScroll3 != newScroll3) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        let initialScroll4 = element.closest(".card-body-area").scrollTop();
        element.closest(".card-body-area").on('scroll', function (e) {

            let newScroll4 = element.closest(".card-body-area").scrollTop();
            if (initialScroll4 != newScroll4) {
                this.setState({ editorEnabled: false });
            }
        }.bind(this));

        element.closest('.layer-custom-fields-container').on('scroll', function (e) {
            this.setState({ editorEnabled: false });
        }.bind(this));

        // if (!this.state.isItemDetails) {
        //     $('body').attr('style', 'overflow: hidden!important');
        // }
    }

    onApplyStyle(){        
        let containerElement = $(this.refs.corgeeConversationSecurityGroups);
        let targetElement = containerElement.find("#select-target-" + this.state.elementId);

        let selectElement = $("#select-render-" + this.state.elementId);
        selectElement.find('.Select').first().attr("style", "position:fixed; top: " + targetElement.offset().top + "px; left: " + targetElement.offset().left+"px");                
                        
        containerElement.closest(".layer-custom-fields-container").first().on("scroll", this.onCloseSelect);
    }

    onCloseSelect(){
        this.setState({ editorEnabled: false });

        //let containerElement = $(this.refs.corgeeConversationSecurityGroups);
        //containerElement.closest(".layer-custom-fields-container").first().off("scroll wheel DOMMouseScroll mousewheel", this.onCloseSelect);
    }

    render() {
        const { canEdit, editorEnabled, selectedIds, selectedNames, dataSecurityGroups, isLoading } = this.state;

        return (
            <div className="click-to-edit" ref="corgeeConversationSecurityGroups">
                {!canEdit && <div className="lbl-non-editable-input">{selectedNames}</div>}
                {canEdit && 
                    (   editorEnabled===true ?
                    <div id={"select-target-" + this.state.elementId} >
                        {/* <RenderToBody> */}
                        <RenderInBody>
                            <div id={"select-render-" + this.state.elementId} className="select-dropdown-container comboBoxRef">
							<ClickOutSide onClickOutSide={this.onBlurSelect} /*whiteList={clickOutSideWhiteList}*/ >
                            <Select ref="editableMultiSelectSecurityGroups"
                                style={{ width: 200 }}
                                value={selectedIds}
                                options={dataSecurityGroups}                                
                                multi={true}                                
                                openOnFocus={true}
                                closeOnSelect={false}
                                clearable={false}
                                onOpen={this.onApplyStyle}
                                autoFocus={true}
                                labelKey={"Name"}
                                valueKey={"GroupId"}
                                onChange={this.onSelectSecurityGroups}
                                onBlur={this.onBlurSelect}
                                optionClassName="Select-option-tether"
                            />
							</ClickOutSide>
                        </div>
                        </ RenderInBody>
                        {/* </RenderToBody> */}
                    </div>
                    :
                    <div className="lbl-editable-input security-group-changed" onClick={this.onEnableEditor}>{selectedNames}</div>                        
                    )
                }
            </div>
        );
    }
}

ConversationSecurityGroups.propTypes = {    
    apiConfig: PropTypes.object.isRequired,    
    itemType: PropTypes.number,
    conversation: PropTypes.object.isRequired,
    disableAutoSave: PropTypes.bool,
    isNew: PropTypes.bool,
    onChangedSecurityGroups: PropTypes.func
};

ConversationSecurityGroups.defaultProps = {
    disableAutoSave: false,
    isNew: false
};

export default ConversationSecurityGroups;